package IUT.server.logic.tables;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;

import org.apache.log4j.Logger;

import IUT.server.logic.model.Loan;
import IUT.utilities.Config;
import IUT.utilities.Trace;
import Scenario.Utility.FireEvent;

public class LoanTable {
	private Logger logger = Trace.getInstance().getLogger("opreation_file");
	List<Loan> loanList = new ArrayList<Loan>();
	Date tempDate;

	private static class LoanListHolder {
		private static final LoanTable INSTANCE = new LoanTable();
	}

	private LoanTable() {
		// set up the default list with some instances
		Loan loan = new Loan(0, "9781442668584", "1", new Date(), "0");
		loanList.add(loan);
		logger.info(String.format("Operation:Initialize LoanTable;LoanTable: %s", loanList));
	};

	public static final LoanTable getInstance() {
		return LoanListHolder.INSTANCE;
	}

	public boolean DoesUserInvalid(int tempUser) {
		return UserTable.getInstance().lookup(tempUser);
	}

	public boolean DoesISBNInvalid(String tempISBN) {
		return TitleTable.getInstance().lookup(tempISBN);
	}

	public boolean DoesCopyNumberInvalid(String tempISBN, String tempCopy) {
		return ItemTable.getInstance().lookup(tempISBN, tempCopy);
	}
	
	public boolean DoesExceedLimit(int tempUser) {
		return LoanTable.getInstance().checkLimit(tempUser);
	}
	
	public boolean HasFee(int tempUser) {
		return FeeTable.getInstance().lookup(tempUser);
	}
	
	public boolean DoesItemAvailable(int tempUser,String tempISBN,String tempCopy) {
		return LoanTable.getInstance().lookup(tempUser, tempISBN, tempCopy);
	}
	
	public boolean DoesLoanExist(int tempUser, String tempISBN, String tempCopy){
		int flag = 0;
		int index = 0;
		for (int i = 0; i < loanList.size(); i++) {
			String ISBN = (loanList.get(i)).getIsbn();
			String copynumber = (loanList.get(i)).getCopynumber();
			int userid = (loanList.get(i)).getUserid();
			if ((userid == tempUser) && ISBN.equalsIgnoreCase(tempISBN) && copynumber.equalsIgnoreCase(tempCopy)) {
				flag = flag + 1;
				index = i;
			} else {
				flag = flag + 0;
			}
		}
		if(flag==0){
			return false;
		}else{
			return true;
    	}
    }
	
	public boolean DoesLoanOverdue(int tempUser, String tempISBN, String tempCopy){
		int index = 0;
		for (int i = 0; i < loanList.size(); i++) {
			String ISBN = (loanList.get(i)).getIsbn();
			String copynumber = (loanList.get(i)).getCopynumber();
			int userid = (loanList.get(i)).getUserid();
			if ((userid == tempUser) && ISBN.equalsIgnoreCase(tempISBN) && copynumber.equalsIgnoreCase(tempCopy)) {
				index = i;
			}
		}
		long time = tempDate.getTime() - loanList.get(index).getDate().getTime();
		boolean flag=false;
		if (time > Config.OVERDUE * Config.STIMULATED_DAY) {
			flag=true;
		}
		return flag;
    }

	public Object createloan(int i, String string, String string2, Date date) {
		tempDate=date;
		FireEvent.FireAnEvent("loanRequested", new ArrayList<Object>(Arrays.asList(i, string, string2, this)));
		String result = "";
		boolean user = UserTable.getInstance().lookup(i);
		boolean isbn = TitleTable.getInstance().lookup(string);
		FireEvent.FireAnEvent("checkDoesISBNInvalid",new ArrayList<Object>());
		boolean copynumber = ItemTable.getInstance().lookup(string, string2);
		FireEvent.FireAnEvent("checkDoesCopyNumberInvalid",new ArrayList<Object>());
		boolean limit = LoanTable.getInstance().checkLimit(i);
		FireEvent.FireAnEvent("checkDoesExceedLimit",new ArrayList<Object>());
		boolean fee = FeeTable.getInstance().lookup(i);
		FireEvent.FireAnEvent("checkHasFee",new ArrayList<Object>());
		boolean oloan = LoanTable.getInstance().lookup(i, string, string2);
		FireEvent.FireAnEvent("checkDoesItemAvailable",new ArrayList<Object>());
		if (user == false) {
			result = "User Invalid";
			FireEvent.FireAnEvent("reportUserInvalid", new ArrayList<Object>());
			logger.info(String.format("Operation:Borrow Item;Loan Info:[%d,%s,%s,%s];State:Fail;Reason:Invalid User.",
					i, string, string2, dateformat(date)));
		} else if (isbn == false) {
			result = "ISBN Invalid";
			FireEvent.FireAnEvent("reportISBNInvalid", new ArrayList<Object>());
			logger.info(String.format("Operation:Borrow Item;Loan Info:[%d,%s,%s,%s];State:Fail;Reason:Invalid ISBN.",
					i, string, string2, dateformat(date)));
		} else if (copynumber == false) {
			result = "Copynumber Invalid";
			FireEvent.FireAnEvent("reportCopyNumberInvalid", new ArrayList<Object>());
			logger.info(
					String.format("Operation:Borrow Item;Loan Info:[%d,%s,%s,%s];State:Fail;Reason:Invalid Copynumber.",
							i, string, string2, dateformat(date)));
		} else {
			if (oloan) {
				if (limit && fee) {
					Loan loan = new Loan(i, string, string2, date, "0");
					loanList.add(loan);
					FireEvent.FireAnEvent("loanAdded", new ArrayList<Object>(Arrays.asList(i, string, string2)));
					result = "success";
					logger.info(String.format("Operation:Borrow Item;Loan Info:[%d,%s,%s,%s];State:Success", i, string,
							string2, dateformat(date)));
				} else if (limit == false) {
					result = "The Maximun Number of Items is Reached";
					FireEvent.FireAnEvent("reportExceedLimit", new ArrayList<Object>());
					logger.info(String.format(
							"Operation:Borrow Item;Loan Info:[%d,%s,%s,%s];State:Fail;Reason:The Maximun Number of Items is Reached.",
							i, string, string2, dateformat(date)));
				} else if (fee == false) {
					result = "Outstanding Fee Exists";
					FireEvent.FireAnEvent("reportHasFee", new ArrayList<Object>());
					logger.info(String.format(
							"Operation:Borrow Item;Loan Info:[%d,%s,%s,%s];State:Fail;Reason:Outstanding Fee Exists.",
							i, string, string2, dateformat(date)));
				}
			} else {
				result = "The Item is Not Available";
				FireEvent.FireAnEvent("reportItemNotAvailable", new ArrayList<Object>());
				logger.info(String.format(
						"Operation:Borrow Item;Loan Info:[%d,%s,%s,%s];State:Fail;Reason:The Item is Not Available.", i,
						string, string2, dateformat(date)));
			}
		}
		FireEvent.FireAnEvent("itemBorrowed", new ArrayList<Object>(Arrays.asList(i, string, string2)));
		return result;
	}

	public boolean lookup(int j, String string, String string2) {
		boolean result = true;
		int flag = 0;
		for (int i = 0; i < loanList.size(); i++) {
			String ISBN = (loanList.get(i)).getIsbn();
			String copynumber = (loanList.get(i)).getCopynumber();
			if (ISBN.equalsIgnoreCase(string) && copynumber.equalsIgnoreCase(string2)) {
				flag = flag + 1;
			} else {
				flag = flag + 0;
			}
		}
		if (flag != 0) {
			result = false;
		}
		return result;
	}

	public boolean checkLimit(int j) {
		boolean result = true;
		int flag = 0;
		for (int i = 0; i < loanList.size(); i++) {
			int userid = (loanList.get(i)).getUserid();
			if (userid == j) {
				flag = flag + 1;
			} else {
				flag = flag + 0;
			}
		}
		if (flag >= Config.MAX_BORROWED_ITEMS) {
			result = false;
		}
		return result;
	}

	public Object renewal(int j, String string, String string2, Date date) {
		tempDate=date;
		FireEvent.FireAnEvent("renewLoanRequested", new ArrayList<Object>(Arrays.asList(j, string, string2, this)));
		String result = "";
		int flag = 0;
		int index = 0;
		boolean limit = LoanTable.getInstance().checkLimit(j);
		boolean fee = FeeTable.getInstance().lookup(j);
		FireEvent.FireAnEvent("checkHasFee",new ArrayList<Object>());
		for (int i = 0; i < loanList.size(); i++) {
			String ISBN = (loanList.get(i)).getIsbn();
			String copynumber = (loanList.get(i)).getCopynumber();
			int userid = (loanList.get(i)).getUserid();
			if ((userid == j) && ISBN.equalsIgnoreCase(string) && copynumber.equalsIgnoreCase(string2)) {
				flag = flag + 1;
				index = i;
			} else {
				flag = flag + 0;
			}
		}
		if (limit && fee) {
			FireEvent.FireAnEvent("checkDoesLoanExist",new ArrayList<Object>());
			if (flag != 0) {
				FireEvent.FireAnEvent("checkDoesExceedRenewLimit",new ArrayList<Object>());
				if (loanList.get(index).getRenewstate().equalsIgnoreCase("0")) {
					loanList.get(index).setUserid(j);
					loanList.get(index).setIsbn(string);
					loanList.get(index).setCopynumber(string2);
					loanList.get(index).setDate(new Date());
					loanList.get(index).setRenewstate("1");
					result = "success";
					FireEvent.FireAnEvent("loanRenewed", new ArrayList<Object>(Arrays.asList(j, string, string2)));
					logger.info(String.format("Operation:Renew Item;Loan Info:[%d,%s,%s,%s];State:Success", j, string,
							string2, dateformat(date)));
				} else {
					result = "Renewed Item More Than Once for the Same Loan";
					FireEvent.FireAnEvent("reportExceedRenewLimit", new ArrayList<Object>());
					logger.info(String.format(
							"Operation:Renew Item;Loan Info:[%d,%s,%s,%s];State:Fail;Reason:Renewed Item More Than Once for the Same Loan.",
							j, string, string2, dateformat(date)));
				}
			} else {
				result = "The loan does not exist";
				FireEvent.FireAnEvent("reportLoanDoesNotExist", new ArrayList<Object>());
				logger.info(String.format(
						"Operation:Renew Item;Loan Info:[%d,%s,%s,%s];State:Fail;Reason:The loan does not exist.", j,
						string, string2, dateformat(date)));
			}

		} else if (limit == false) {
			result = "The Maximun Number of Items is Reached";
			FireEvent.FireAnEvent("reportExceedLimit", new ArrayList<Object>());
			logger.info(String.format(
					"Operation:Renew Item;Loan Info:[%d,%s,%s,%s];State:Fail;Reason:The Maximun Number of Items is Reached.",
					j, string, string2, dateformat(date)));
		} else if (fee == false) {
			result = "Outstanding Fee Exists";
			FireEvent.FireAnEvent("reportHasFee", new ArrayList<Object>());
			logger.info(String.format(
					"Operation:Renew Item;Loan Info:[%d,%s,%s,%s];State:Fail;Reason:Outstanding Fee Exists.", j, string,
					string2, dateformat(date)));
		}
		FireEvent.FireAnEvent("loanRenewedCompeleted", new ArrayList<Object>(Arrays.asList(j, string, string2)));
		return result;
	}

	private String dateformat(Date date) {
		DateFormat format1 = new SimpleDateFormat("yyyy-MM-dd hh:mm:ss");
		String datestr = format1.format(date);
		return datestr;
	}

	public Object returnItem(int j, String string, String string2, Date date) {
		tempDate=date;
		FireEvent.FireAnEvent("returnLoanRequested", new ArrayList<Object>(Arrays.asList(j, string, string2, this)));
		String result = "";
		int flag = 0;
		int index = 0;
		for (int i = 0; i < loanList.size(); i++) {
			String ISBN = (loanList.get(i)).getIsbn();
			String copynumber = (loanList.get(i)).getCopynumber();
			int userid = (loanList.get(i)).getUserid();
			if ((userid == j) && ISBN.equalsIgnoreCase(string) && copynumber.equalsIgnoreCase(string2)) {
				flag = flag + 1;
				index = i;
			} else {
				flag = flag + 0;
			}
		}
		if (flag != 0) {
			FireEvent.FireAnEvent("checkDoesLoanOverdue",new ArrayList<Object>());
			long time = date.getTime() - loanList.get(index).getDate().getTime();
			loanList.remove(index);
			FireEvent.FireAnEvent("loanDeleted", new ArrayList<Object>(Arrays.asList(j, string, string2)));
			logger.info(String.format("Operation:Return Item;Loan Info:[%d,%s,%s,%s];State:Success", j, string, string2,
					dateformat(date)));
			if (time > Config.OVERDUE * Config.STIMULATED_DAY) {
				FeeTable.getInstance().applyfee(j, time);
			}
			result = "success";
			FireEvent.FireAnEvent("loanDeletionCompleted", new ArrayList<Object>(Arrays.asList(j, string, string2)));
		} else {
			result = "The Loan Does Not Exist";
			FireEvent.FireAnEvent("reportLoanDoesNotExist", new ArrayList<Object>());
			logger.info(String.format(
					"Operation:Return Item;Loan Info:[%d,%s,%s,%s];State:Fail;Reason:The Loan Does Not Exist.", j,
					string, string2, dateformat(date)));
		}
		FireEvent.FireAnEvent("itemReturned", new ArrayList<Object>(Arrays.asList(string, string2)));
		return result;
	}

	public List<Loan> getLoanTable() {
		return loanList;
	}

	public boolean looklimit(int j) {
		boolean result = true;
		int flag = 0;
		for (int i = 0; i < loanList.size(); i++) {
			int userid = (loanList.get(i)).getUserid();
			if (userid == j) {
				flag = flag + 1;
			} else {
				flag = flag + 0;
			}
		}
		if (flag != 0) {
			result = false;
		}
		return result;
	}

	public boolean checkUser(int j) {
		boolean result = true;
		int flag = 0;
		for (int i = 0; i < loanList.size(); i++) {
			int userid = (loanList.get(i)).getUserid();
			if (userid == j) {
				flag = flag + 1;
			} else {
				flag = flag + 0;
			}
		}
		if (flag != 0) {
			result = false;
		}
		return result;
	}

	public boolean checkLoan(String string, String string2) {
		boolean result = true;
		int flag = 0;
		for (int i = 0; i < loanList.size(); i++) {
			String ISBN = (loanList.get(i)).getIsbn();
			String copynumber = (loanList.get(i)).getCopynumber();
			if (ISBN.equalsIgnoreCase(string) && copynumber.equalsIgnoreCase(string2)) {
				flag = flag + 1;
			} else {
				flag = flag + 0;
			}
		}
		if (flag != 0) {
			result = false;
		}
		return result;
	}

	public boolean checkLoan(String string) {
		boolean result = true;
		int flag = 0;
		for (int i = 0; i < loanList.size(); i++) {
			String ISBN = (loanList.get(i)).getIsbn();
			if (ISBN.equalsIgnoreCase(string)) {
				flag = flag + 1;
			} else {
				flag = flag + 0;
			}
		}
		if (flag != 0) {
			result = false;
		}
		return result;
	}
}
