package MonitoringFramework.ACLClass;
import java.io.IOException;
import java.lang.reflect.Constructor;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Observable;
import java.util.Observer;
import java.util.concurrent.ConcurrentHashMap;

import MonitoringFramework.edu.carleton.scs.jacl.vf.engine.ContractManager;
import Scenario.Utility.StringUtils;
import MonitoringFramework.ACLClass.ACLEventProcessor.EventProcessorState;
import MonitoringFramework.classtrn.BlackBoard;
import MonitoringFramework.classtrn.LogManager;
import MonitoringFramework.classtrn.PrintToScreen;
import MonitoringFramework.classtrn.ScenarioThreadFactory;
import MonitoringFramework.classtrn.TraceElement;


public abstract class ACLContract implements Observer, ACLEventListener
{
	protected long OID;
	protected Map<String, List<ACLScenario>> scenarioInstanceMap; //Each contract holds all the  scenario instances.

	protected Date timestamp; // FixedVariable Type
	protected Object userdefined;
	
	public abstract void setBindpoint (Object instance); 
	public abstract Object getBindpoint () ;   

	protected ACLContract () {
		OID = ((ContractManager) ContractManager.getInstance()).getNextContractID();		
	}
	
	public long getContractID () {
		return OID;
	}
	
	@Override
	public long getListenerId(){
		return getContractID();
		
	}
	
	
	//Of no use now.
	@Override
	public void update(Observable arg, Object obj) {
		if (obj instanceof Date)
		{
			timestamp = (Date) obj;
		}
		else 
		if (obj instanceof ACLEvent)
		{
			//
		}
	}

   // Pre, Post and Checks

	private TraceElement traceElement;
	
	// This would allow us to do some checks and logging on the PreSet Conditions
	protected Integer PreSet (Integer condition) {
		return condition;
	}
	
	protected void Execute () {
		String message = String.format( "Returning Call Back From %s", traceElement.toString());	
		LogManager.getInstance().debug(this, message);
	}
	
	protected void Execute (Boolean PreCheck) {
		if (PreCheck) {
			String message = String.format( "Returning Call Back From %s", traceElement.toString());	
			LogManager.getInstance().debug(this, message);
		}
	}
	
	protected void Pre (Boolean condition) { Pre(condition,""); }
	
	protected void Post (Boolean condition) { Post(condition,""); }
	
	protected void Pre (Boolean condition, String belief) {
		if (traceElement != null) {
			String message
		    = String.format( "%s: %s %s %s", "Executing Pre Condition", belief, traceElement.toString(), condition ? " - passed" : " - failed" );
			LogManager.getInstance().debug(this, message);
		}
		else {
			LogManager.getInstance().debug(this, "Executing Post Condition: Trace element not set");
		}
	}
		
	
	protected void Post (Boolean condition, String belief) {
		if (traceElement != null) {
			String message 
			= String.format("%s: %s %s %s", "Executing Post Condition", belief, traceElement.toString(), condition ? " - passed" : " - failed" );
			LogManager.getInstance().debug(this, message);
		}
		else {
			LogManager.getInstance().debug(this, "Executing Post Condition: Trace element not set");
		}
	}
	
	protected void Check (Boolean condition, Object o, String belief) {
		String message 
		     = String.format( "Contract [%s] instance [%s] Belief [%s] %s " , o.getClass().getSimpleName()
				                      										, o.hashCode(), belief, condition ? "- passed" : "- failed");
		LogManager.getInstance().debug(this, message);
	}

	protected void setTraceElement (Object instance) {
		traceElement = new TraceElement (this,instance);
	}
	

	protected void printStackTrace (StackTraceElement[] elements) {
		for (StackTraceElement e: elements) {
			System.out.printf("\nClass name %s Method %s LineNumber %d HashCode %d\n"
			,e.getClassName()
			,e.getMethodName()
			,e.getLineNumber()
			,e.getClass().hashCode());
		}
	}
	
	protected void pause () {
		try {	System.in.read();	} 
		catch (IOException e) { e.printStackTrace();	}
	}
	
	/**
	 * Key- Scenario Value - A list of scenario instances.
	 * @return the scenarioInstanceMap
	 */
	protected Map<String, List<ACLScenario>> getScenarioInstanceMap() {
		if(this.scenarioInstanceMap==null){
			scenarioInstanceMap = new ConcurrentHashMap<String, List<ACLScenario>>(); //let's just use concurrentHashMap. Need to come back to this.
			//I think it is safe. It is being accessed only from the eventNotifier thread. Which does all that needs to be done in one call.
		}
		return scenarioInstanceMap;
	}
	
	@Override
	public ACLEventListenertype getACLEventListenerType() {
		// TODO Auto-generated method stub
		return ACLEventListenertype.ACLCONTRACT;
		
	}
	
	/**
	 * Will put the calling thread to sleep if events are still being processed!
	 * For Now prints on screen. Can be switched to log to a file.
	 * Called from Oracle on every ACLContractInstance that is created.
	 */
	public void getScenarioInstanceStatusAfterIUTExit(){
		//wait for all the scenario thread to complete their execution
		ScenarioThreadFactory.getInstance().shutDownAndWaitForScenaioThreadCompletion();
			//Any scenario still in created state must be set to false state.
			getScenarioInstanceMap().forEach((event,scenarioInstanceList) -> 
			scenarioInstanceList.forEach((x)-> 
			{
				if(x.getEventProcesorState().equals(EventProcessorState.CREATED)){
					PrintToScreen.printToScreen("setting scenrios in created state to fail after all threads exit");
					x.setProcessorStatusToFail();
				}
				//Now print the scenarioName, InstanceID, TriggeringEventName and the state of scenario.
				System.out.println("ScenarioName: "+ x.getListenerName()+ "InstanceId: "+ x.getListenerId()+ " Triggering Event: "+ x.getTriggeringEvent()+ " final state: " 
				+ x.getEventProcesorState());
				
			}));		
	}	
	
	
	public abstract List<String> getExpectedEventNames();
	
	@Override
	public  void registerForACLEvents(){
		List<String> ge=getExpectedEventNames();
		getExpectedEventNames().forEach((eventName) -> BlackBoard.registerForACLEventNotification(eventName, this));	
	}
	
	public abstract Map<String, List<String>> getScenarioTriggerMap();
	
	protected ACLScenario getScenarioInstanceByName(String scenarioName, ACLContract ownerContact, ACLEventType triggeringEvent ){
		ACLScenario scenarioInstance;
		try{
			Class<?> clazz = Class.forName(StringUtils.getFullyQualifiedScenarioClassName(scenarioName));
			Constructor<?> constructor = clazz.getConstructor(ACLContract.class, ACLEventType.class); //we are getting the constructor with this signature (ACLContract ownerContract,ACLEventType trigger) 
			scenarioInstance = (ACLScenario) constructor.newInstance(ownerContact, triggeringEvent);
		}
		catch(Exception ex){
			ex.printStackTrace();
			PrintToScreen.printErrorToScreen("failed to create instace of the scenario: "+ scenarioName+ "for the event: "+ triggeringEvent);
			return null; //there is no graceful way of dealing with this. This should never happen!!
		}
		return scenarioInstance;
	}
	
	
	public void notifyOfACLEvent(ACLEventType event) {
		List<ACLScenario> scenarioInstances;
		if(getScenarioTriggerMap().containsKey(event.getEventName())){ //this check will read from map
			for(String scenarioName: getScenarioTriggerMap().get(event.getEventName())){ //instantiate all the scenarios that need to be instantiated
				ACLScenario scenarioInstance =  getScenarioInstanceByName(scenarioName, this, event);
				scenarioInstance.registerForACLEvents(); //Do I need to keep a handle of all the scenario instances ??
				//start the scenario thread
				ScenarioThreadFactory.getInstance().runScenarioThread(scenarioInstance);
				//save the scenarioInstance in scenarioInstanceMap.
				if(getScenarioInstanceMap().get(scenarioName)==null){ //This is the first scenario Instance.
					scenarioInstances = new ArrayList<ACLScenario>();
					scenarioInstances.add(scenarioInstance);
					getScenarioInstanceMap().put(scenarioName, scenarioInstances);
				}
				else{
					//append
					getScenarioInstanceMap().get(scenarioName).add(scenarioInstance);
				}
			}
		}		
	}
	/* (non-Javadoc)
	 * @see java.lang.Object#hashCode()
	 */
	/* (non-Javadoc)
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + (int) (OID ^ (OID >>> 32));
		return result;
	}
	
	
	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (!(obj instanceof ACLContract)) {
			return false;
		}
		ACLContract other = (ACLContract) obj;
		if (OID != other.OID) {
			return false;
		}
		return true;
	}
	
	
}
	


