package MonitoringFramework.ACLClass;

import java.util.EventObject;
import java.util.concurrent.atomic.AtomicBoolean;


public class ACLEvent extends EventObject implements ACLEventType {
	/**
	 * 
	 */
	private static final long serialVersionUID = -4748505076251003431L;

	private static Integer TOTAL_NUMBER_OF_EVENTS=0;

	private final Long timeStamp;
	private final String eventName;
	private final Integer eventId;
	private boolean isUniquelyObservable =false; //by default false;
	private AtomicBoolean hasBeenObserved = new AtomicBoolean(false);//when observed, swtich to true.

	
	public ACLEvent(Object eventsource,String eventName, long eventCreationTime) {
		super(eventsource);
		this.eventName = eventName;
		this.timeStamp = eventCreationTime;
		this.eventId = ++ACLEvent.TOTAL_NUMBER_OF_EVENTS;
	}

	/**
	 * Using this constructor is like creating a new event from another event. the constructed event is a fresh new event which has not been observed yet.
	 * The created event also inherits the id of the supplied event.
	 * @param event
	 */
	public ACLEvent(ACLEventType event) {
		super(event.getSource());
		this.eventName = event.getEventName();
		this.timeStamp = event.getTimeStamp();
		this.eventId = event.geteventId();
		this.isUniquelyObservable = event.isUnqiuelyObservable();
	}

	/* (non-Javadoc)
	 * @see classtrn.ACLEventinterface#getTimeStamp()
	 */
	@Override
	public final long getTimeStamp() {
		return timeStamp;
	}

	/* (non-Javadoc)
	 * @see classtrn.ACLEventinterface#getEventName()
	 */
	@Override
	public String getEventName() {
		return eventName;
	}
	
	/* (non-Javadoc)
	 * @see classtrn.ACLEventinterface#geteventId()
	 */
	@Override
	public Integer geteventId(){
		return eventId;
	}
	
	/* (non-Javadoc)
	 * @see classtrn.ACLEventinterface#getSource()
	 */
	
	@Override
	public Object getSource(){
		return super.getSource();
	}

	
	@Override
	public String toString() {
		return this.getEventName();
	}

	@Override
	public boolean hasParameter() {
		return false;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((eventName == null) ? 0 : eventName.hashCode());
		return result;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (!(obj instanceof ACLEvent)) {
			return false;
		}
		ACLEvent other = (ACLEvent) obj;
		if (eventName == null) {
			if (other.eventName != null) {
				return false;
			}
		} else if (!eventName.equals(other.eventName)) {
			return false;
		}
		return true;
	}


	/**
	 * @param isUniquelyObserved the isUniquelyObserved to set
	 */
	protected void setUniquelyObservable(boolean isUniquelyObserved) {
		this.isUniquelyObservable = isUniquelyObserved;
	}

	@Override
	public boolean isUnqiuelyObservable() {
		return this.isUniquelyObservable;
	}

	@Override
	public boolean hasBeenObseved() {		
		return this.hasBeenObserved.get();
	}

	@Override
	public void Observe() {
		this.hasBeenObserved.compareAndSet(false, true);		
	}

	@Override
	public boolean canBeObserved() {
		if(!this.isUniquelyObservable){//is not to be treated as uniquely observable and mutliple tasks/expression can process the same event to see if they recieved it in order.
			return true;
		}
		else{
			//event is uniquely observable
			if(hasBeenObseved()){
				//and has already been observed then it can't be observed anymore
				return false;
			}
			else{
				// it has not been observed 
				return true;
			}
		}
	}





}
