package MonitoringFramework.ACLClass;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.script.ScriptEngine;
import javax.script.ScriptEngineManager;
import javax.script.ScriptException;

import org.apache.commons.lang3.text.StrBuilder;

import MonitoringFramework.classtrn.Constants;
import MonitoringFramework.classtrn.EventToObserve;
import MonitoringFramework.classtrn.HasEventOccured;
import Scenario.Utility.StringUtils;
import MonitoringFramework.classtrn.Constants.Booleans;

public class ACLEventExpression implements ACLEventProcessor {
	private org.apache.commons.lang3.text.StrBuilder booleanExpression; //Not threadsafe either. Will decide later
	private org.apache.commons.lang3.text.StrBuilder booleanExpressionVars;// ="professorRetired=false;professorDied=false;";
	private Map<String, HasEventOccured> eventObservanceMap;
	private List<String> expectedEvents;
	private Set<String> forbiddenEvents;
	private EventProcessorState expressionState;
	private ACLEventProcessor ownerTask;
	private static final ScriptEngineManager SCRIPTMANAGER = new ScriptEngineManager();
	private static final ScriptEngine SCRIPTENGINE = SCRIPTMANAGER.getEngineByName(Constants.JAVASCRIPTENGINE);
	

//	public enum ACLEventExpresionState{
//		CREATED,SET,FAILED,PASSED; //You can't fail an expression because you don't know if you are waiting for some event to happen or not.
//	}
	
	public ACLEventExpression(ScenarioTask task,EventProcessorState state) {
		eventObservanceMap= new HashMap<>(); 
		expectedEvents = new ArrayList<>();
		forbiddenEvents = new HashSet<>();
		this.ownerTask= task;
		this.expressionState = state;
		booleanExpression = new StrBuilder();
		booleanExpressionVars = new StrBuilder();
	}
	
	public Map<String, HasEventOccured> getEventObservanceMap() {
		return eventObservanceMap;
	}
	/**
	 * @return the booleanExpression
	 */
	public org.apache.commons.lang3.text.StrBuilder getBooleanExpression() {
		return booleanExpression;
	}

	
	public ACLEventExpression observeEvent(EventToObserve event){   
		eventObservanceMap.put(event.getEventToObserve(), new HasEventOccured(event.getEventOccuranceCount(),event.getOverflowAllowed()));
		booleanExpressionVars.append(event.getEventToObserve()).append("=false;"); //declaring the variable. all the events by default set to false for expression.
		booleanExpression.append("("+ event.getEventToObserve()+")");
		return this; 
	}
	
	public ACLEventExpression or(EventToObserve event){
		eventObservanceMap.put(event.getEventToObserve(), new HasEventOccured(event.getEventOccuranceCount(),event.getOverflowAllowed()));
		booleanExpressionVars.append(event.getEventToObserve()).append("=false;"); //declaring the variable. all the events by default set to false for expression.
		booleanExpression.append(Constants.Booleans.OR.getActualSymbolValue()).append("("+event.getEventToObserve()+")");   
		return this;
	}
	
	public ACLEventExpression or(ACLEventExpression expression){
		this.eventObservanceMap.putAll(expression.getEventObservanceMap());
		merge(expression.getBooleanExpression(), Constants.Booleans.OR);
		expression.getEventObservanceMap().forEach((key, value) -> booleanExpressionVars.append(key).append("=false;")); 
		return this;
	}
	
	public ACLEventExpression and(ACLEventExpression expression){
		this.eventObservanceMap.putAll(expression.getEventObservanceMap());
		merge(expression.getBooleanExpression(), Constants.Booleans.AND);
		expression.getEventObservanceMap().forEach((key, value) -> booleanExpressionVars.append(key).append("=false;")); 
		return this;
	}
	
	public ACLEventExpression and(EventToObserve event){
		eventObservanceMap.put(event.getEventToObserve(), new HasEventOccured(event.getEventOccuranceCount(),event.getOverflowAllowed()));
		booleanExpressionVars.append(event.getEventToObserve()).append("=false;"); //declaring the variable. all the events by default set to false for expression.
		booleanExpression.append(Constants.Booleans.AND.getActualSymbolValue()).append("("+event.getEventToObserve()+")");   
		return this;
	}
	
	public ACLEventExpression notObserve(String eventName){
		this.forbiddenEvents.add(eventName);
//		booleanExpressionVars.append(eventName).append("=false;"); //declaring the variable. all the events by default set to false for expression.
//		booleanExpression.append(Constants.Booleans.AND.getActualSymbolValue()).append("("+eventName+")"); 
		return this;
	}
	
	
	private void merge(org.apache.commons.lang3.text.StrBuilder toBeMerged, Constants.Booleans booleanVal){
		//System.out.println("Boolean being asked: " + booleanVal + "Actual value: "+ booleanVal.getActualSymbolValue());
		this.booleanExpression.insert(0,"(").append(")").append(booleanVal.getActualSymbolValue()).append("(").append(toBeMerged).append(")");
	}
	
	private Boolean evaluate(){
		Boolean result= false;
		
		 try {
		  //get the latest results before evaluating the expression	
	      //make a copy of the buffer and use that locally.
		  String booleanExpressionCopy = booleanExpression.toString();
		  for(String event: getEventObservanceMap().keySet()){
			  HasEventOccured eventOccuranceInformation = getEventObservanceMap().get(event);
			  booleanExpressionCopy = booleanExpressionCopy.replaceAll(event, eventOccuranceInformation.checkIfEventOccured().toString());
		  }
/*		  this.getEventObservanceMap().forEach(
				  (key,value) ->  booleanExpressionCopy.replaceAll(key, value.checkIfEventOccured().toString()) );*/
		  //PrintToScreen.printToScreen("About to evaluate: "+ booleanExpressionVars.toString()+booleanExpressionCopy.toString());
		  result = StringUtils.parseStringAsBooleanWithFalseAsDefault(ACLEventExpression.SCRIPTENGINE.eval(booleanExpressionVars.toString()+booleanExpressionCopy).toString());
		} catch (ScriptException e) {
			result = false;
			e.printStackTrace();
		}
		 return result;
	}
	
	@Override
	public EventProcessorState getEventProcesorState() {
		if(this.expressionState.equals(EventProcessorState.PASSED)){
			return this.expressionState; //if it has already passed then no need to re-evaluate.
		}
		if(this.evaluate()){
			expressionState = EventProcessorState.PASSED;
		}			
		return expressionState;
	}
	
	@Override
	public void processEvent(ACLEventType event){
		//check if it exists in the eventMap
		if(this.getEventObservanceMap().containsKey(event.getEventName()) && event.canBeObserved() && !expressionState.equals(EventProcessorState.PASSED)){
			//check the state. If it is in 'CREATED' state then that means event has been received out of order.
			if(expressionState.equals(EventProcessorState.CREATED)){
				//set the owner status to fail.				
				 //the only way an expression reaches a failed state is when an event is received out of order.
				setProcessorStatusToFail();
			}
			else{
				this.getEventObservanceMap().get(event.getEventName()).reportEventOccurance(event);
			}
		}
		else if( event.canBeObserved() && this.forbiddenEvents.contains(event.getEventName())){
			setProcessorStatusToFail();
		}
	}
	
	@Override
	public void setProcessorStatusToFail(){
		getParentEventProcessor().setProcessorStatusToFail();
		expressionState = EventProcessorState.FAILED;
	}
	
	@Override
	public void setUpToAcceptEvents(){
		this.expressionState =  EventProcessorState.SET;
	}
	
	@Override	
	public List<ACLEventProcessor> getConstituentsEventProcessors(){
		//A place holder as this is the bottom most eventProcessor. No call should come here..
		return null;
	}
	
	@Override
	public ACLEventProcessor getParentEventProcessor(){
		return this.ownerTask;
	}
	
	@Override
	public void skip() {
		this.expressionState = EventProcessorState.PASSED; //Skipped expression are to be treated as passed.		
	}

	@Override
	public List<String> getExpectedEvents() {
		if(expectedEvents.isEmpty()){
			eventObservanceMap.keySet().forEach((eventName) -> expectedEvents.add(eventName)) ;
		}
		if(expectedEvents.size()!= eventObservanceMap.keySet().size()){
			//just clear and re populate
			expectedEvents.clear();
			eventObservanceMap.keySet().forEach((eventName) -> expectedEvents.add(eventName)) ;
		}
		return expectedEvents;
	}

	@Override
	public Set<String> getForbiddenEvents() {
		return forbiddenEvents;
	}
	
}
