package MonitoringFramework.ACLClass;

import MonitoringFramework.classtrn.ConditionToBeChecked;
import MonitoringFramework.classtrn.ScenarioTaskType;

public class ChoiceAlternativeTask extends ScenarioTask {
	private ACLEventProcessor choiceTask; //can be ACLEventExpression/Another Task(Static or Dynamic)
	private ACLEventProcessor alternativeTask;
	private ConditionToBeChecked condition;
	
	public ChoiceAlternativeTask(ACLScenario owner, Integer taskid,ConditionToBeChecked condition) {		
		super(owner, ScenarioTaskType.Dynamic, taskid);
		this.condition = condition; //if the condition evaluates to true when the first event is 'received' then choice ask will be notified of the task and alternative task will be marked as skipped		
	}

	/**
	 * @return the choiceTask
	 */
	public ACLEventProcessor getChoiceTask() {
		return choiceTask;
	}

	/**
	 * @param choiceTask the choiceTask to set
	 */
	public void setChoiceTask(ACLEventProcessor choiceTask) {
		this.choiceTask = choiceTask;
		super.addACLEventProcessorToTask(choiceTask);
	}

	/**
	 * @return the alternativeTask
	 */
	public ACLEventProcessor getAlternativeTask() {
		return alternativeTask;
	}

	/**
	 * @param alternativeTask the alternativeTask to set
	 */
	public void setAlternativeTask(ACLEventProcessor alternativeTask) {
		this.alternativeTask = alternativeTask;
		super.addACLEventProcessorToTask(alternativeTask);
	}

	/**
	 * @return the condition
	 */
	public ConditionToBeChecked getCondition() {
		return condition;
	}	
	
	@Override
	public void setUpToAcceptEvents(){
		this.taskStatus = EventProcessorState.SET;		
		//set the first eventExpression to accept events
		//this.getConstituentsEventProcessors().get(0).setUpToAcceptEvents(); //instead of calling set up on the first element in the list , 
		//it has to be selective - dependent on the condition evaluation
		/*
		 * Evaluate the condition and select the effective task - setUp that selected task to receive events.
		 */
		if(this.condition.getConditionResult()){
			//Choice task becomes the effective task and will be set up.
			this.choiceTask.setUpToAcceptEvents();
			this.alternativeTask.skip();
			//if the skipped task has forbidden events they are not forbidden anymore, remove them from forbidden list
			super.getForbiddenEvents().removeAll(this.alternativeTask.getForbiddenEvents());
			this.alternativeTask.getForbiddenEvents().clear();
		}
		else{
			//Alternative task becomes the effective task and will be set up.
			this.alternativeTask.setUpToAcceptEvents();
			this.choiceTask.skip();
			super.getForbiddenEvents().removeAll(this.choiceTask.getForbiddenEvents());
			this.choiceTask.getForbiddenEvents().clear();
		}
		
		//this.getConstituentsEventProcessors().get(0).setUpToAcceptEvents();
	}
	

}
