package MonitoringFramework.ACLClass;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import MonitoringFramework.classtrn.ObservabilityInformation;
import MonitoringFramework.classtrn.PrintToScreen;
import MonitoringFramework.classtrn.ReflectionUtility;
import MonitoringFramework.classtrn.ScenarioTaskType;
//@zhibo, treat the for loops as a loop of parallel task
public class ForTask extends ScenarioTask {
	
	private ObservabilityInformation observabilityInformation;	
	private final Map<String, Integer> parametrIndexMap;
	private Integer numberOfForTaskConstituents;
	private final List<ACLEventProcessor> forConstituents;
	private List<?> expectedParameterList;
	
	public ForTask(ACLScenario owner,Integer taskId, ObservabilityInformation observabilityInfo) {
		super(owner, ScenarioTaskType.Dynamic, taskId);
		this.observabilityInformation = observabilityInfo;
		parametrIndexMap = new HashMap<>();
		numberOfForTaskConstituents = 0;
		forConstituents = new ArrayList<>();
	}
	
	public ObservabilityInformation getObservabilityInformation() {
		return observabilityInformation;
	}
	
	public void setTaskToBeRepeated(ACLEventProcessor task, Integer index){
		forConstituents.add(new ForTaskConstituents(this, task, expectedParameterList.get(index))); 
	}
	
	public void addEventParameterIndexInformation(String eventName, Integer parameterIndex){
		parametrIndexMap.put(eventName, parameterIndex);
	}
	
	public Integer getNumberOfForTaskConstituents(){
		Object contractInstance = ReflectionUtility.resolveContractInstance(observabilityInformation);
		String observabilityName = observabilityInformation.getObservabilityInformation().getLeft();
		Object obsReturnValue = ReflectionUtility.getObservabilityReturnValue(observabilityName, contractInstance);
		if(obsReturnValue instanceof List<?> ){
			expectedParameterList = (List<?>) obsReturnValue; 
			numberOfForTaskConstituents = expectedParameterList.size();
		}
		else{
			numberOfForTaskConstituents = 0;
			PrintToScreen.printErrorToScreen("Expected the observability: "+ observabilityName+" to return list type but recieved: "+ obsReturnValue.getClass().getTypeName());
		}
		return numberOfForTaskConstituents;
	}

	@Override
	public void setUpToAcceptEvents(){
		this.taskStatus = EventProcessorState.SET;		
		super.addACLEventProcessorsToTask(forConstituents);
		this.getConstituentsEventProcessors().forEach((contituent) -> contituent.setUpToAcceptEvents());	

	}


	public Map<String, Integer> getParameterIndexMap() {
		return parametrIndexMap;
	}


}