
package MonitoringFramework.ACLClass;

import java.util.ArrayList;
import java.util.List;

public class ParameterizedACLEvent implements ACLEventType {
	
	private final ACLEventType event;
	private List<Object> parameterValues;
	//private ACLParameter parameter;
	
	public ParameterizedACLEvent(ACLEventType event, Object value) {
		this.event = event;
		parameterValues = new ArrayList<>();
		this.parameterValues.add(value)	;
	}
	
	public ParameterizedACLEvent(ACLEventType event, List<Object> values) {
		this.event = event;
		parameterValues = new ArrayList<>();
		this.parameterValues.addAll(values)	;	
	}
	
	//some sort of check has to be put in to place to make sure that this is only of either String, Integer or Contract type
	//rename to getParameterValues
	public List<Object> getParameterValue() {
		return parameterValues;
	}
	
	public ACLEventType getEvent() {
		return event;
	}
	
	
	
	
	@Override
	public String toString() {
		String eventString = event.toString();		
		this.parameterValues.forEach((parameter)-> eventString.concat(parameter.toString()));
		return eventString;
	}
	@Override
	public long getTimeStamp() {
		return getEvent().getTimeStamp();
	}
	@Override
	public String getEventName() {
		return getEvent().getEventName();
	}
	@Override
	public Integer geteventId() {
		return getEvent().geteventId();
	}
	@Override
	public Object getSource() {
		return getEvent().getSource();
	}
	@Override
	public boolean hasParameter() {		
		return true;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((event == null) ? 0 : event.hashCode());
		result = prime * result + ((parameterValues == null) ? 0 : parameterValues.hashCode());
		return result;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (!(obj instanceof ParameterizedACLEvent)) {
			return false;
		}
		ParameterizedACLEvent other = (ParameterizedACLEvent) obj;
		if (event == null) {
			if (other.event != null) {
				return false;
			}
		} else if (!event.equals(other.event)) {
			return false;
		}
		if (parameterValues == null) {
			if (other.parameterValues != null) {
				return false;
			}
		} else if (!parameterValues.equals(other.parameterValues)) {
			return false;
		}
		return true;
	}

	@Override
	public boolean isUnqiuelyObservable() {		
		return this.event.isUnqiuelyObservable();
	}

	@Override
	public boolean hasBeenObseved() {
		return this.event.hasBeenObseved();
	}

	@Override
	public void Observe() {
		this.event.Observe();
	}

	@Override
	public boolean canBeObserved() {
		return this.event.canBeObserved();
	}

	
}
