package MonitoringFramework.ACLClass;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import MonitoringFramework.classtrn.PrintToScreen;
import MonitoringFramework.classtrn.ScenarioTaskType;

//later  should be abstracted out 
public abstract class ScenarioTask implements ACLEventProcessor{ 
	private ScenarioTaskType scenarioTaskType;
	protected EventProcessorState taskStatus = EventProcessorState.CREATED; //by default task status is created
	private List<ACLEventProcessor> constituentEventProcessors;
	

	protected Integer taskID; ///keeping it integer. Number of tasks in a scenario shouldn't cross the allowed integer limit.
	protected ACLEventProcessor ownerScenario;
	protected List<String> expectedEvents;
	protected Set<String> forbiddenEvents; //may be these should be sets
	

	protected ScenarioTask(ACLEventProcessor owner,ScenarioTaskType scenarioTaskType, Integer taskId) {
		this.ownerScenario = owner;
		this.scenarioTaskType = scenarioTaskType;
		this.taskStatus = EventProcessorState.CREATED; //just created.
		this.taskID = taskId;
		constituentEventProcessors = new ArrayList<>();
		expectedEvents = new ArrayList<>();
		forbiddenEvents= new HashSet<>();
	}

	protected ScenarioTaskType getScenarioTaskType() {
		return scenarioTaskType;
	}

	protected void setScenarioTaskType(ScenarioTaskType scenarioTaskType) {
		this.scenarioTaskType = scenarioTaskType;
	}
	
	@Override
	public List<String> getExpectedEvents(){		
		return expectedEvents;
	}
	
	@Override
	public EventProcessorState getEventProcesorState() {
		if(taskStatus.equals(EventProcessorState.CREATED) || taskStatus.equals(EventProcessorState.FAILED)){
			//if task has not been SET(i.e. is in created state) then it is just in waiting(any task that is not in set state will turn to fail state upon receiving any event)
			//if it is in failed state, that is the final state for this task and can be returned right away
			return taskStatus;
		}
		else{
			//if it is in SET state then re-evaluate the state before returning the result.
			//EventProcessorState state=this.getConstituentsEventProcessors().get(0).getEventProcesorState();
			if(this.getConstituentsEventProcessors().stream().allMatch(
					task-> task.getEventProcesorState().equals(EventProcessorState.PASSED))){
				this.taskStatus = EventProcessorState.PASSED;
			}
		}
		return taskStatus;
	}

	@Override
	public void setProcessorStatusToFail() {
		this.taskStatus = EventProcessorState.FAILED;
		//the task has failed. Set the owner scenario's status to fail as well.
		this.getParentEventProcessor().setProcessorStatusToFail();
	}
	
	@Override
	public void setUpToAcceptEvents(){
		this.taskStatus = EventProcessorState.SET;   
		//set the first eventExpression to accept events
		this.getConstituentsEventProcessors().get(0).setUpToAcceptEvents();
	}
	
	public long getId(){
		return taskID;
	}
	
	@Override
	public List<ACLEventProcessor> getConstituentsEventProcessors() {
		return constituentEventProcessors;
	}
	
	//May ne change this to addAClEventProcessorToTask
	public void addACLEventProcessorToTask(ACLEventProcessor eventExpression) {
		if (eventExpression != null) {
			this.constituentEventProcessors.add(eventExpression);
			this.getExpectedEvents().addAll(eventExpression.getExpectedEvents());
			//also check for forbidden events
			this.getForbiddenEvents().addAll(eventExpression.getForbiddenEvents());
		}
	}
	
	/**
	 * Adds the event in the same order as they appear in the provided list
	 * @param eventExpressionList
	 */
	public void addACLEventProcessorsToTask(List<ACLEventProcessor> eventExpressionList) {
		if (eventExpressionList != null) {
			eventExpressionList.forEach(
					(eventExpression) -> addACLEventProcessorToTask(eventExpression)) ;
		}
	} 
	
	/**
	 * Processes the receives event and floods it to expression, if need be.
	 * @param event
	 */
	@Override
	public void processEvent(ACLEventType event){
		if(this.getExpectedEvents().contains(event.getEventName()) && event.canBeObserved() && !taskStatus.equals(EventProcessorState.PASSED)){
			//check the state. If it is in 'CREATED' state then that means event has been received out of order.
			if(taskStatus.equals(EventProcessorState.CREATED)){
				//set the owner status to fail.
				PrintToScreen.printErrorToScreen("Will set the owner scenario to fail.Called by one of the tasks.Event: "+ event.getEventName()+" recieved out of order");
				this.setProcessorStatusToFail();
			}
			else{
				Boolean didCurrentExpressionSucceed = false;
				for(int i=0; i<getConstituentsEventProcessors().size() ; i++){
					int nextExpressionIndex = i+1;
					ACLEventProcessor currentEventExpression = getConstituentsEventProcessors().get(i);
					didCurrentExpressionSucceed = currentEventExpression.getEventProcesorState().equals(EventProcessorState.PASSED);
					if(didCurrentExpressionSucceed && nextExpressionIndex < getConstituentsEventProcessors().size() ){ //The first task at 0 indices must have been set up to accept events already
						ACLEventProcessor nextTask = getConstituentsEventProcessors().get(nextExpressionIndex);
						if(nextTask.getEventProcesorState().equals(EventProcessorState.CREATED)){
							nextTask.setUpToAcceptEvents();
						}
					}					
				}
				this.getConstituentsEventProcessors().forEach((eventExpression) -> eventExpression.processEvent(event));
			}
		}
		else if(event.canBeObserved() && this.forbiddenEvents.contains(event.getEventName())){
			setProcessorStatusToFail();
		}
	}

	/* (non-Javadoc)
	 * @see classtrn.ACLEventProcessor#getParentEventProcessor()
	 */
	@Override
	public ACLEventProcessor getParentEventProcessor() {
		return ownerScenario;
	}

	/* (non-Javadoc)
	 * @see classtrn.ACLEventProcessor#skip()
	 */
	@Override
	public void skip() {
		this.taskStatus = EventProcessorState.PASSED;		
	}

	
	@Override
	public Set<String> getForbiddenEvents() {
		return forbiddenEvents;
	}
	

}
