package MonitoringFramework.classtrn;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.List;


public class ConditionToBeChecked {

	public enum ObservabilityType {
		VALUETYPE, BOOLEANTYPE;
	}

	public enum Operator {		
		Equalto, NotEqualTo, GreaterThan, GreaterThanOrEqualTo, LessThan, LessThanOrEqualTo;
		// its for the parser to not allow the GreaterThan,	// GreaterThanOrEqualTo, LessThan, LessThanOrEqualTo to be used with // ContractInstance.
	}
	
	private Object valueToBeComparedAgainst; // this can either be a contract// instance, String, Integer or// a ScenarioParameter which// resolves to any of three	// types.
	private ObservabilityInformation observabilityInformation;
	private Operator operator;
	private Boolean conditionResult=false; //This may be foolish but I hate NPE.
	private Boolean hasConditionbeenEvaluated = false; // Will there be threading issues ? If being accessed from multiple threads come back and change to AtomicBoolean and so on..
	
	public ConditionToBeChecked(ObservabilityInformation observabilityInformation,Object valueToBeComparedAgainst,Operator operator) {
		super();
		this.valueToBeComparedAgainst = valueToBeComparedAgainst;
		this.observabilityInformation = observabilityInformation;
		this.operator = operator;
	}


	/**
	 * @return the valueToBeComparedAgainst
	 */
	public Object getValueToBeComparedAgainst() {
		return valueToBeComparedAgainst;
	}


	/**
	 * @return the observabilityInformation
	 */
	public ObservabilityInformation getObservabilityInformation() {
		return observabilityInformation;
	}


	/**
	 * @return the operator
	 */
	public Operator getOperator() {
		return operator;
	}

	/**
	 * @return the conditionResult
	 */
	public Boolean getConditionResult() {
		if(!hasConditionbeenEvaluated ){
			evaluateCondition();
		}
		return conditionResult;
	}
	
	private void evaluateCondition(){
		//first calculate the Actual value using the observability
		//to do that, we need the contract instance.
		
		//@zhibo, modify to evaluate Condition
		//Object contractInstance= observabilityInformation.getObservabilityInformation().getRight();
		//need to resolve the scenario parameter class into contract instance
		Object contractInstance = ReflectionUtility.resolveContractInstance(observabilityInformation);
		String observabilityName=observabilityInformation.getObservabilityInformation().getLeft();
		List<Object> observabilityParamaters=observabilityInformation.getParameters();
		//find the method
		Method observabilityMethod = null;
		Object actualResult=null;
		try {
			if(observabilityParamaters.size()==0){
				observabilityMethod = contractInstance.getClass().getMethod(observabilityName, new Class[] {});
			}else{
				Method[] methods = contractInstance.getClass().getMethods();
				Object[] paramaters = observabilityParamaters.toArray();
				for(Method method:methods){
					if(method.getName().equals(observabilityName) && method.getParameterCount()==paramaters.length){
						observabilityMethod=method;
					}
					
				}
				
			}
		} catch (NoSuchMethodException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (SecurityException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		try {
			//need to consider the situation that the observibility methods have paramaters
			if(observabilityParamaters.size()==0){
				actualResult = observabilityMethod.invoke(contractInstance, new Object[]{});
			}else{
				Object[] paramaters = observabilityParamaters.toArray();
				Class<?>[] paramatersClasses = observabilityMethod.getParameterTypes();
				Object[] invokingParmaters=new Object[paramatersClasses.length];
				for(int i=0;i<paramatersClasses.length;i++){
					ScenarioParameter sp=(ScenarioParameter) paramaters[i];
					invokingParmaters[i]=sp.getValue();
				}
			    actualResult = observabilityMethod.invoke(contractInstance, invokingParmaters);
			    		   
			    		   
			}
		} catch (IllegalAccessException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IllegalArgumentException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (InvocationTargetException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		//resolve the condition
		ObservabilityType obsType = observabilityInformation.getObservabilityType();
		if(obsType.equals(ObservabilityType.BOOLEANTYPE)){
			if(actualResult.equals(valueToBeComparedAgainst)){
				conditionResult=true;
			}
		}else if(obsType.equals(ObservabilityType.VALUETYPE)){
			evalForValueReturnTypeObservability(observabilityName, actualResult);	
		}
		System.out.println("check condition:"+observabilityMethod+ ", actual result:" + actualResult+", expected result:"+valueToBeComparedAgainst);

		
	}


	private void evalForValueReturnTypeObservability(String observabilityName, Object actualValue) {
		switch (this.operator) {
		case Equalto:
			conditionResult = actualValue.equals(valueToBeComparedAgainst);
			break;
		case NotEqualTo:
			conditionResult = !(actualValue.equals(valueToBeComparedAgainst));
			break;
		case GreaterThan:
			// Should be applied only to Integers!!! //So the check is that
			// actual value is compared against the expected value!!
			try {
				conditionResult = ((Integer) actualValue) > (Integer) valueToBeComparedAgainst;
			} catch (Exception e) {
				PrintToScreen.printErrorToScreen("Expected integer but recieved :"
						+ actualValue.getClass().getTypeName() + "while evaluating condition for observability: "
						+ observabilityName + " setting codition result to false");
				conditionResult = false;
			}
			break;
		case GreaterThanOrEqualTo:
			// Should be applied only to Integers!!! //So the check is that
			// actual value is compared against the expected value!!
			try {
				conditionResult = ((Integer) actualValue) >= (Integer) valueToBeComparedAgainst;
			} catch (Exception e) {
				PrintToScreen.printErrorToScreen("Expected integer but recieved :"
						+ actualValue.getClass().getTypeName() + "while evaluating condition for observability: "
						+ observabilityName + " setting codition result to false");
				conditionResult = false;
			}
			break;
		case LessThan:
			// Should be applied only to Integers!!! //So the check is that
			// actual value is compared against the expected value!!
			try {
				conditionResult = ((Integer) actualValue) < (Integer) valueToBeComparedAgainst;
			} catch (Exception e) {
				PrintToScreen.printErrorToScreen("Expected integer but recieved :"
						+ actualValue.getClass().getTypeName() + "while evaluating condition for observability: "
						+ observabilityName + " setting codition result to false");
				conditionResult = false;
			}
			break;
		case LessThanOrEqualTo:
			// Should be applied only to Integers!!! //So the check is that
			// actual value is compared against the expected value!!
			try {
				conditionResult = ((Integer) actualValue) <= (Integer) valueToBeComparedAgainst;
			} catch (Exception e) {
				PrintToScreen.printErrorToScreen("Expected integer but recieved :"
						+ actualValue.getClass().getTypeName() + "while evaluating condition for observability: "
						+ observabilityName + " setting codition result to false");
				conditionResult = false;
			}
			break;
		default:
			break;
		}
	}


	private void evalForBooleanReturnTypeObservability(String observabilityName, Object actualValue) {
		try{
			conditionResult  = (Boolean) actualValue; //What if its not a boolean ? No check made.	
		}
		catch(Exception e){
			PrintToScreen.printErrorToScreen("Expected boolean from :"+ observabilityName+" but recieved some other type. Setting the condition result to false");
			conditionResult = false;
			e.printStackTrace();
		}
	}

	

}
