package MonitoringFramework.classtrn;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import org.apache.commons.lang3.tuple.Pair;

import MonitoringFramework.ACLClass.ACLContract;
import Scenario.Utility.StringUtils;

/**
 * 
 * @author hneedham
 * The class should be enhanced to include support for the parameters that come from owner contract
 */
public class ScenarioParameter {
	
	private Object value; 
	
	private Pair<String, ScenarioParameter> observabilityInformation; //Observability and Parameter pair.
	
	public ScenarioParameter(Object value) {
		this.value = value;
	}
	
	public ScenarioParameter(Object value, Pair<String, ScenarioParameter> observabilityInfo) {
		this.value = value;
		this.observabilityInformation = observabilityInfo;
	}
	/**
	 * It may return List or a scalar type. The caller method must check for return type and use contains() or equal() as applicable. 
	 * @return
	 */
	public Object getParameterValue() {
		if(this.value==null){
			String observability = observabilityInformation.getLeft();
			Object contractInstanceVar = this.observabilityInformation.getRight().getParameterValue(); //This call to getValue will result in recursion-like call.
			if(StringUtils.isNotEmpty(observability) && contractInstanceVar!=null){
				if(contractInstanceVar.getClass().getSuperclass().isAssignableFrom(ACLContract.class)){
					value = ReflectionUtility.getObservabilityReturnValue(observability, contractInstanceVar);
				}
			}
		}
		return value;
	}
	

	public void setValue(Object value) {
		PrintToScreen.printToScreen("Setting the parameterValue to: "+ value.toString());
		this.value = value;
	}
	
	public Object getValue(){
		return this.value;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((getParameterValue() == null) ? 0 : value.hashCode());
		return result;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (!(obj instanceof ScenarioParameter)) {
			return false;
		}
		ScenarioParameter other = (ScenarioParameter) obj;
		if (this.getParameterValue() == null) {
			if (other.getParameterValue() != null) {
				return false;
			}
		} else if (!this.getParameterValue().equals(other.getParameterValue())) {
			return false;
		}
		return true;
	}

}
