package Scenario;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import org.apache.commons.collections4.list.SetUniqueList;
import org.apache.commons.lang3.tuple.Pair;
import MonitoringFramework.ACLClass.ACLContract;
import MonitoringFramework.ACLClass.ACLEventExpression;
import MonitoringFramework.ACLClass.ACLEventType;
import MonitoringFramework.ACLClass.ForTask;
import MonitoringFramework.ACLClass.ScenarioTask;
import MonitoringFramework.classtrn.EventToObserve;
import MonitoringFramework.classtrn.ObservabilityInformation;
import MonitoringFramework.classtrn.ScenarioParameter;
import MonitoringFramework.classtrn.StaticScenarioTaskType;
import Scenario.MyAnnotation.MyAnnotations.Scenario;
import MonitoringFramework.ACLClass.ChoiceAlternativeTask;
import MonitoringFramework.classtrn.ConditionToBeChecked;
import MonitoringFramework.classtrn.ObservabilityInformation;
import MonitoringFramework.classtrn.ConditionToBeChecked.ObservabilityType;
import MonitoringFramework.ACLClass.ParallelTask;
import MonitoringFramework.classtrn.ConditionToBeChecked.Operator;

@Scenario(name = "AddTitle", owner = "TitleTableContract", trigger = "titleAdditionRequested")
public class AddTitleScenario extends MonitoringFramework.ACLClass.ACLScenario {
	public static final String SELFNAME = "AddTitleScenario";
	public static final String TRIGGERNAME = "titleAdditionRequested";
	public static final Boolean HASPARAMETERS = true;
	public static final List<String> EXPECTEDEVENTNAMES;

	public AddTitleScenario(ACLContract ownerContract, ACLEventType trigger) {
		super(ownerContract, trigger);
	}

	static {
		List<String> expectedEvents = new ArrayList<String>();
		expectedEvents.add("informTitleAlreadyExists");
		expectedEvents.add("newTitleCreated");
		expectedEvents.add("titleAdditionCompleted");
		EXPECTEDEVENTNAMES = SetUniqueList.setUniqueList(expectedEvents);
	}

	@Override
	protected void initializeParameterMap() {
		parameterMap.put(TRIGGERNAME, new ArrayList<>(
				Arrays.asList(new ScenarioParameter(null), new ScenarioParameter(null), new ScenarioParameter(null))));
		parameterMap.put("newTitleCreated", new ArrayList<>(
				Arrays.asList(parameterMap.get(TRIGGERNAME).get(0), parameterMap.get(TRIGGERNAME).get(1))));
		parameterMap.put("titleAdditionCompleted",
				new ArrayList<>(Arrays.asList(parameterMap.get(TRIGGERNAME).get(0))));
	}

	@Override
	protected void initializeTasksForScenario() {
		ConditionToBeChecked DoesTitleExistCondition = new ConditionToBeChecked(
				new ObservabilityInformation(Pair.of("DoesTitleExist", parameterMap.get(TRIGGERNAME).get(2)),
						new ArrayList<Object>(Arrays.asList(parameterMap.get(TRIGGERNAME).get(0))),
						ObservabilityType.BOOLEANTYPE),
				true, Operator.Equalto);
		ScenarioTask st1 = new ChoiceAlternativeTask(this, 1, DoesTitleExistCondition);
		((ChoiceAlternativeTask) st1).setChoiceTask(new ACLEventExpression(st1, EventProcessorState.CREATED)
				.observeEvent(new EventToObserve("informTitleAlreadyExists", 1)));
		((ChoiceAlternativeTask) st1).setAlternativeTask(new ACLEventExpression(st1, EventProcessorState.CREATED)
				.observeEvent(new EventToObserve("newTitleCreated", 1))
				.and(new EventToObserve("titleAdditionCompleted")));
		getConstituentsEventProcessors().add(st1);
	}

	@Override
	public String getListenerName() {
		return SELFNAME;
	}

	@Override
	protected Map<String, List<ScenarioParameter>> getParameterMap() {
		return this.parameterMap;
	}

	@Override
	protected Boolean scenarioHasParameterizedEvents() {
		return HASPARAMETERS;
	}

	@Override
	public List<String> getExpectedEvents() {
		return EXPECTEDEVENTNAMES;
	}

	@Override
	protected String getExpectedTriggeringEventName() {
		return TRIGGERNAME;
	}
}
