package Scenario;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import org.apache.commons.collections4.list.SetUniqueList;
import org.apache.commons.lang3.tuple.Pair;
import MonitoringFramework.ACLClass.ACLContract;
import MonitoringFramework.ACLClass.ACLEventExpression;
import MonitoringFramework.ACLClass.ACLEventType;
import MonitoringFramework.ACLClass.ForTask;
import MonitoringFramework.ACLClass.ScenarioTask;
import MonitoringFramework.classtrn.EventToObserve;
import MonitoringFramework.classtrn.ObservabilityInformation;
import MonitoringFramework.classtrn.ScenarioParameter;
import MonitoringFramework.classtrn.StaticScenarioTaskType;
import Scenario.MyAnnotation.MyAnnotations.Scenario;
import MonitoringFramework.ACLClass.ChoiceAlternativeTask;
import MonitoringFramework.classtrn.ConditionToBeChecked;
import MonitoringFramework.classtrn.ObservabilityInformation;
import MonitoringFramework.classtrn.ConditionToBeChecked.ObservabilityType;
import MonitoringFramework.ACLClass.ParallelTask;
import MonitoringFramework.classtrn.ConditionToBeChecked.Operator;

@Scenario(name = "RemoveUser", owner = "UserTableContract", trigger = "userDeletionRequested")
public class RemoveUserScenario extends MonitoringFramework.ACLClass.ACLScenario {
	public static final String SELFNAME = "RemoveUserScenario";
	public static final String TRIGGERNAME = "userDeletionRequested";
	public static final Boolean HASPARAMETERS = true;
	public static final List<String> EXPECTEDEVENTNAMES;

	public RemoveUserScenario(ACLContract ownerContract, ACLEventType trigger) {
		super(ownerContract, trigger);
	}

	static {
		List<String> expectedEvents = new ArrayList<String>();
		expectedEvents.add("informUserNotExist");
		expectedEvents.add("checkUserHasLoan");
		expectedEvents.add("informUserHasLoan");
		expectedEvents.add("checkUserHasFee");
		expectedEvents.add("informUserHasFee");
		expectedEvents.add("deleteUser");
		expectedEvents.add("userDeleted");
		expectedEvents.add("userDeletionCompleted");
		EXPECTEDEVENTNAMES = SetUniqueList.setUniqueList(expectedEvents);
	}

	@Override
	protected void initializeParameterMap() {
		parameterMap.put(TRIGGERNAME,
				new ArrayList<>(Arrays.asList(new ScenarioParameter(null), new ScenarioParameter(null))));
		parameterMap.put("checkUserHasLoan", new ArrayList<>(Arrays.asList(parameterMap.get(TRIGGERNAME).get(0))));
		parameterMap.put("checkUserHasFee", new ArrayList<>(Arrays.asList(parameterMap.get(TRIGGERNAME).get(0))));
		parameterMap.put("deleteUser", new ArrayList<>(Arrays.asList(parameterMap.get(TRIGGERNAME).get(0))));
		parameterMap.put("userDeleted", new ArrayList<>(Arrays.asList(parameterMap.get(TRIGGERNAME).get(0))));
		parameterMap.put("userDeletionCompleted", new ArrayList<>(Arrays.asList(parameterMap.get(TRIGGERNAME).get(0))));
	}

	@Override
	protected void initializeTasksForScenario() {
		ConditionToBeChecked DoesUserExistCondition = new ConditionToBeChecked(
				new ObservabilityInformation(Pair.of("DoesUserExist", parameterMap.get(TRIGGERNAME).get(1)),
						new ArrayList<Object>(Arrays.asList(parameterMap.get(TRIGGERNAME).get(0))),
						ObservabilityType.BOOLEANTYPE),
				false, Operator.Equalto);
		ScenarioTask st1 = new ChoiceAlternativeTask(this, 1, DoesUserExistCondition);
		((ChoiceAlternativeTask) st1).setChoiceTask(new ACLEventExpression(st1, EventProcessorState.CREATED)
				.observeEvent(new EventToObserve("informUserNotExist", 1)));
		((ChoiceAlternativeTask) st1).setAlternativeTask(new ACLEventExpression(st1, EventProcessorState.CREATED)
				.observeEvent(new EventToObserve("checkUserHasLoan", 1)));
		getConstituentsEventProcessors().add(st1);
		ConditionToBeChecked UserHasLoanCondition = new ConditionToBeChecked(
				new ObservabilityInformation(Pair.of("UserHasLoan", parameterMap.get(TRIGGERNAME).get(1)),
						new ArrayList<Object>(Arrays.asList(parameterMap.get(TRIGGERNAME).get(0))),
						ObservabilityType.BOOLEANTYPE),
				true, Operator.Equalto);
		ScenarioTask st2 = new ChoiceAlternativeTask(this, 2, UserHasLoanCondition);
		((ChoiceAlternativeTask) st2).setChoiceTask(new ACLEventExpression(st2, EventProcessorState.CREATED)
				.observeEvent(new EventToObserve("informUserHasLoan", 1)));
		((ChoiceAlternativeTask) st2).setAlternativeTask(new ACLEventExpression(st2, EventProcessorState.CREATED)
				.observeEvent(new EventToObserve("checkUserHasFee", 1)));
		getConstituentsEventProcessors().add(st2);
		ConditionToBeChecked UserHasFeeCondition = new ConditionToBeChecked(
				new ObservabilityInformation(Pair.of("UserHasFee", parameterMap.get(TRIGGERNAME).get(1)),
						new ArrayList<Object>(Arrays.asList(parameterMap.get(TRIGGERNAME).get(0))),
						ObservabilityType.BOOLEANTYPE),
				true, Operator.Equalto);
		ScenarioTask st3 = new ChoiceAlternativeTask(this, 3, UserHasFeeCondition);
		((ChoiceAlternativeTask) st3).setChoiceTask(new ACLEventExpression(st3, EventProcessorState.CREATED)
				.observeEvent(new EventToObserve("informUserHasFee", 1)));
		((ChoiceAlternativeTask) st3).setAlternativeTask(new ACLEventExpression(st3, EventProcessorState.CREATED)
				.observeEvent(new EventToObserve("deleteUser", 1)).and(new EventToObserve("userDeleted"))
				.and(new EventToObserve("userDeletionCompleted")));
		getConstituentsEventProcessors().add(st3);
	}

	@Override
	public String getListenerName() {
		return SELFNAME;
	}

	@Override
	protected Map<String, List<ScenarioParameter>> getParameterMap() {
		return this.parameterMap;
	}

	@Override
	protected Boolean scenarioHasParameterizedEvents() {
		return HASPARAMETERS;
	}

	@Override
	public List<String> getExpectedEvents() {
		return EXPECTEDEVENTNAMES;
	}

	@Override
	protected String getExpectedTriggeringEventName() {
		return TRIGGERNAME;
	}
}
