package Scenario.Utility;
import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.collections4.list.SetUniqueList;

public class StringUtils {
	public static String SCENARIOPACKAGENAME="Scenario"; //need to be replaced with actual package name. 
	
	public static Boolean parseStringAsBooleanWithFalseAsDefault(String toBeParsed) {
		return isEmpty(toBeParsed)?false: toBeParsed.equalsIgnoreCase("true") ;
	}
	
	public static Boolean isNotEmpty(String str){
		return str==null?false:str.length()>0;
	}
	
	public static Boolean isEmpty(String str){
		return str==null?true:str.length()<=0;
	}
	
	public static String getFullyQualifiedScenarioClassName(String scenarioName){
		return SCENARIOPACKAGENAME+"."+scenarioName+"Scenario";
	}
	
	//@zhibo, find all the existing scenario class
	public static List<Class<?>> getFullScenarioList(){
		ScenarioUtils runner = new ScenarioUtils();
		Field[] names=runner.getClass().getFields();
		List<Class<?>> scenarioList=new ArrayList<Class<?>>();
		for(Field field: names){
			String className = null;
			try {
				className = field.get(field)+"Scenario";
				try {
					Class<?> clazz = Class.forName(SCENARIOPACKAGENAME+"."+className);
					if(clazz!=null){
						scenarioList.add(clazz);
					}
				} catch (ClassNotFoundException e) {}
			} catch (IllegalArgumentException | IllegalAccessException e1) {}
		}
		return SetUniqueList.setUniqueList(scenarioList);
	}
	
	//@zhibo,find all the existing scenario belongs to one specific contract
	public static List<Class<?>> getScenarioListByContractName(String contract){
		List<Class<?>> scenarioList=StringUtils.getFullScenarioList();
		List<Class<?>> resultList=new ArrayList<Class<?>>();
		for(int i=0;i<scenarioList.size();i++){
			Class<?> clazz=scenarioList.get(i);
			//we still need to figure out a better way to handle the annotations
			String scenarioAnnos=clazz.getAnnotations()[0].toString();
			String owner=StringUtils.findScenarioOwner(scenarioAnnos);
			
			if(owner.equals(contract)){
				resultList.add(clazz);
			}
		}
		return resultList;
	}
	
	//@zhibo,find all the trigger events for the existing scenario belongs to one specific contract
	public static List<String> getAllTriggerEvents(List<Class<?>> scenarioList){
		List<String> resultList=new ArrayList<String>();
		for(int i=0;i<scenarioList.size();i++){
			Class<?> clazz=scenarioList.get(i);
			String scenarioAnnos=clazz.getAnnotations()[0].toString();
			String triggerEvent=StringUtils.findScenarioTrigger(scenarioAnnos);
			resultList.add(triggerEvent);
		}	
		return resultList;
	}
	
	//@zhibo,find all the existing scenario belongs to one specific trigger  event for one contract
		public static List<String> getAllScenarioNames(String triggerEvent,String contract){
			List<Class<?>> scenarioList=getScenarioListByContractName(contract);
			List<String> resultList=new ArrayList<String>();
			for(int i=0;i<scenarioList.size();i++){
				Class<?> clazz=scenarioList.get(i);
				String scenarioAnnos=clazz.getAnnotations()[0].toString();
				String triggerEventstr=StringUtils.findScenarioTrigger(scenarioAnnos);
				if(triggerEventstr.equals(triggerEvent)){
					String scenarioName=StringUtils.findScenarioName(scenarioAnnos);
					resultList.add(scenarioName);
				}
			}	
			return resultList;
		}
	
	//@zhibo,a brute-force way to get the name attribute from the annotations
	public static String findScenarioName(String scenarioAnnos){
		String subString="name=";
		String name = "";
		int i=scenarioAnnos.lastIndexOf(subString);
		for(int j=i+subString.length();j<scenarioAnnos.length();j++){
			if(scenarioAnnos.charAt(j)!=','){
				name=name+scenarioAnnos.charAt(j);
			}else{
				break;
			}
		}
		return name;
		
	}
	//@zhibo,a brute-force way to get the owner attribute from the annotations
	public static String findScenarioOwner(String scenarioAnnos){
		String subString="owner=";
		String owner = "";
		int i=scenarioAnnos.lastIndexOf(subString);
		for(int j=i+subString.length();j<scenarioAnnos.length();j++){
			if(scenarioAnnos.charAt(j)!=','){
				owner=owner+scenarioAnnos.charAt(j);
			}else{
				break;
			}
		}
		return owner;	
	}
	//@zhibo,a brute-force way to get the trigger attribute from the annotations
	public static String findScenarioTrigger(String scenarioAnnos){
		String subString="trigger=";
		String trigger = "";
		int i=scenarioAnnos.lastIndexOf(subString);
		for(int j=i+subString.length();j<scenarioAnnos.length();j++){
			if(scenarioAnnos.charAt(j)!=')'){
				trigger=trigger+scenarioAnnos.charAt(j);
			}else{
				break;
			}
		}
		return trigger;	
	}
	
	 
}
