package controller;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;
import java.util.Properties;
import java.util.Scanner;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import Utils.ConstantsUtil;

public class EventBindingController {
	public static final String currentDir=System.getProperty("user.dir");
	public static final File metadataDir=new File(currentDir,"/resource/metadata");
	public static final File eventFile=new File(metadataDir,"firingEvents.properties");
	public static final Properties prop = new Properties();
	
	private static final ConfigController configController=new ConfigController();
	
	public File initialize() throws IOException{
		if(!eventFile.exists()){
			eventFile.createNewFile();
		}
		return eventFile;
	}
	
	public void deleteAll(){
		eventFile.delete();
	}
	
	//get all classes
	public void getAllClasses(File directory){
		if(directory.exists()){
		File[] fileList=directory.listFiles();
		for (int i = 0; i < fileList.length; i++) {
		      if (fileList[i].isFile() && fileList[i].getName().endsWith(".java")) {
		    	  try(FileOutputStream fos=new FileOutputStream(this.initialize())){
		    		String className=buildNameHelper(fileList[i].getAbsolutePath());
		  			prop.setProperty(className, buildPropertyHelper(className));
		  			prop.store(fos, "Event Binding");
		  		} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
		      } else if (fileList[i].isDirectory()) {
		    	  getAllClasses(fileList[i]);
		      }
		}
		}
	}
	
	public String buildNameHelper(String path){
		String parentPath=new File(configController.read(ConstantsUtil.IUTDirectory)).getAbsolutePath();
		String className="";
		className=path.substring(parentPath.length()+1,path.length()-5).replace("\\", ".");
		return className;
	}
	
	public String buildPathHelper(String className){
		return className.replace(".", "\\")+".java";
	}
	//get all methods from each class
	public List<String> getAllMethods(String className){
		File classFile=new File(configController.read(ConstantsUtil.IUTDirectory),buildPathHelper(className));
		Scanner scanner = null;
		try {
			scanner = new Scanner(classFile);
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		List<String> methodList=new ArrayList<String>();
		while (scanner.hasNextLine()) {
		   String lineFromFile = scanner.nextLine();
		   Matcher m=Pattern.compile("((public|private).*\\(.*\\))").matcher(lineFromFile);
		   if(m.find() && !(lineFromFile.substring(lineFromFile.indexOf("p"), lineFromFile.indexOf("(")).contains("="))){
			   String methodName=m.group(0);
			   methodList.add(methodName);
		   }
		}
		return methodList;
		
	}
	
	//get all defined events in each method
	public List<String> getAllFireEvents(String className, String methodName){
		File classFile=new File(configController.read(ConstantsUtil.IUTDirectory),buildPathHelper(className));
		Scanner scanner = null;
		try {
			scanner = new Scanner(classFile);
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		List<String> eventList=new ArrayList<String>();
		boolean flag=false;
		while (scanner.hasNextLine()) {
		   String lineFromFile = scanner.nextLine();
		   if(lineFromFile.contains(methodName)){
			   flag=true;
		   }
		   if(lineFromFile.contains("FireEvent") && flag){
			   //check if commented out
			   if(!lineFromFile.trim().startsWith("//")){
				   String eventName=lineFromFile.substring(lineFromFile.indexOf("\"")+1,lineFromFile.indexOf(",")-1);
				   eventList.add(eventName);
			   }
		   }
		   if((lineFromFile.contains("public ") || lineFromFile.contains("private ")) && (!lineFromFile.contains(methodName)) && lineFromFile.contains("(")){
			   flag=false;
		   }
		}
		return eventList;
	}
	
	public String getFireEventImplementation(String className, String eventName){
		String result="";
		File classFile=new File(configController.read(ConstantsUtil.JScenarioDirectory),buildPathHelper(className));
		Scanner scanner = null;
		try {
			scanner = new Scanner(classFile);
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		while (scanner.hasNextLine()) {
		   String lineFromFile = scanner.nextLine();
		   if(lineFromFile.contains("FireEvent") && lineFromFile.contains(eventName)){
			   //check if commented out
			   if(!lineFromFile.trim().startsWith("//")){
				   result=lineFromFile;
				   break;
			   }
		   }
		}
		return result;
	}
	
	public String buildPropertyHelper(String className){
		List<String> methodList=getAllMethods(className);
		String methodStr="";
		for(int i=0;i<methodList.size();i++){
			List<String> eventList=getAllFireEvents(className,methodList.get(i));
			String eventStr="";
			if(eventList.size()>0){
			for(int j=0;j<eventList.size();j++){
				eventStr=eventStr+eventList.get(j)+",";
			}
			eventStr=eventStr.substring(0, eventStr.length()-1);
			}
			methodStr=methodStr+methodList.get(i)+":"+eventStr+";";
		}
		return methodStr;
	}
	
	
	public List<String> getAllProperty(){
		List<String> properties=new ArrayList<String>();
		try(FileInputStream fis=new FileInputStream(this.initialize())){
			prop.load(fis);
			Enumeration<?> e = prop.propertyNames();
		    while (e.hasMoreElements()) {
		    	properties.add((String) e.nextElement());
		    }
		} catch (FileNotFoundException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		} catch (IOException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
		return properties;
	}
	
	public Object[][] getTable(String className){
		Object[][] result=null;
		try(FileInputStream fis=new FileInputStream(this.initialize())){
			prop.load(fis);
			String methodStr=prop.getProperty(className);
			String[] methodList=methodStr.split(";");
			result=new Object[methodList.length][2];
			for(int i=0;i<methodList.length;i++){
				String[] subMethodStr=methodList[i].split(":");
				if(subMethodStr.length>1){
					result[i]=new Object[]{subMethodStr[0],subMethodStr[1]};
				}else{
					result[i]=new Object[]{subMethodStr[0],""};
				}
			}
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return result;
	}
	
	public List<String> getMethodList(String className){
		List<String> methodStrList=new ArrayList<String>();
		try(FileInputStream fis=new FileInputStream(this.initialize())){
			prop.load(fis);
			String methodStr=prop.getProperty(className);
			String[] methodList=methodStr.split(";");
			for(String method:methodList){
				String[] subMethodStr=method.split(":");
				methodStrList.add(subMethodStr[0]);
			}
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return methodStrList;
	}
	
	public List<String> getEventList(){
		List<String> eventList=new ArrayList<String>();
		try(FileInputStream fis=new FileInputStream(this.initialize())){
			prop.load(fis);
			Enumeration<?> e = prop.propertyNames();
		    while (e.hasMoreElements()) {
		    	String methodStr=prop.getProperty((String) e.nextElement());
		    	String[] methodList=methodStr.split(";");
				for(String method:methodList){
					String[] subMethodStr=method.split(":");
					if(subMethodStr.length>1){
						String[] eventStrList=subMethodStr[1].split(",");
						if(eventStrList.length>0){
							for(String event:eventStrList){
								eventList.add(event);
							}
						}
					}
				}
		    	
		    }
		} catch (FileNotFoundException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		} catch (IOException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
		return eventList;
	}
	
	public void saveFile(String className,String file){
		try(FileWriter writer =new FileWriter(new File(configController.read(ConstantsUtil.IUTDirectory),buildPathHelper(className)), false)) {
			writer.write(file);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		//update the property files
		getAllClasses(new File(configController.read(ConstantsUtil.IUTDirectory)));
	}
	
	public String getClassContent(String className) {
	    try(BufferedReader br = new BufferedReader(new FileReader(new File(configController.read(ConstantsUtil.IUTDirectory),buildPathHelper(className))))){
	        StringBuilder sb = new StringBuilder();
	        String line = br.readLine();
	        while (line != null) {
	            sb.append(line);
	            sb.append("\n");
	            line = br.readLine();
	        }
	        return sb.toString();
	    } catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return "";
		}
	}
	
}
