package controller;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Properties;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import Utils.ConstantsUtil;

public class ScenarioBindingController {
	public static final String currentDir = System.getProperty("user.dir");
	public static final File metadataDir = new File(currentDir, "/resource/metadata");
	public static final File scenarioDir = new File(currentDir, "/resource/scenarios");
	public static final File scenarioFile = new File(metadataDir, "scenarios.properties");
	public static final Properties prop = new Properties();

	public File initialize() throws IOException {
		if (!scenarioFile.exists()) {
			scenarioFile.createNewFile();
		}
		return scenarioFile;
	}
	
	public void deleteScenarios(){
		for(File f: scenarioDir.listFiles()){
			  f.delete(); 
		}
	}
	
	public void deleteAll(){
		scenarioFile.delete();
		deleteScenarios();
	}

	public void createScenario(String scenarioName, String className, String editorContent, boolean status) {
		writePropertyHelper(className, buildPropertyHelper(scenarioName, className, status));
		saveScenario(scenarioName,className,editorContent);

	}

	public String buildPropertyHelper(String scenarioName, String className, boolean status) {
		String result = "";
		try (FileInputStream fis = new FileInputStream(this.initialize())) {
			prop.load(fis);
			if (!(prop.getProperty(className) == null)) {
				result = prop.getProperty(className);
				result = result + scenarioName + ":" + status + ";";
			} else {
				result = scenarioName + ":" + status + ";";
			}
			return result;
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return "";
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return "";
		}
	}

	public void writePropertyHelper(String key, String value) {
		try (FileOutputStream fos = new FileOutputStream(this.initialize())) {
			if(value != null && !value.isEmpty()){
				prop.setProperty(key, value);
			}else{
				prop.remove(key);
			}
			prop.store(fos, "Scenario Config");
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public void saveScenario(String scenarioName,String className, String editorContent) {
		String metadata="@scenarioName="+scenarioName+",\n"+"@className="+className+";\n";
		try (FileWriter writer = new FileWriter(new File(scenarioDir, scenarioName + ".txt"), false)) {
			writer.write(metadata+editorContent);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public boolean checkScenarioName(String scenarioName) {
		boolean flag = false;
		File tempFile = new File(scenarioDir, scenarioName + ".txt");
		if (tempFile.exists()) {
			flag = true;
		}
		return flag;
	}

	public Object[][] getTable(String className) {
		Object[][] result = null;
		try (FileInputStream fis = new FileInputStream(this.initialize())) {
			prop.load(fis);
			String scenarioStr = prop.getProperty(className);
			if (scenarioStr != null) {
				String[] scenarioList = scenarioStr.split(";");
				if (scenarioList.length > 0) {
					result = new Object[scenarioList.length][2];
					for (int i = 0; i < scenarioList.length; i++) {
						String[] status = scenarioList[i].split(":");
						if (status.length > 1) {
							if (status[1].equals("true")) {
								result[i] = new Object[] { status[0], "On" };
							} else {
								result[i] = new Object[] { status[0], "Off" };
							}
						}
					}
				}
			}
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return result;
	}

	public boolean getScenarioStatus(String className, String scenarioName) {
		boolean flag = false;
		try (FileInputStream fis = new FileInputStream(this.initialize())) {
			prop.load(fis);
			if (!(prop.getProperty(className) == null)) {
				String scenarioStr = prop.getProperty(className);
				if (scenarioStr != null) {
					String[] scenarioList = scenarioStr.split(";");
					if (scenarioList.length > 0) {
						for (int i = 0; i < scenarioList.length; i++) {
							String[] status = scenarioList[i].split(":");
							if (status.length > 1) {
								if (status[0].equals(scenarioName) && status[1].equals("true")) {
									flag = true;
									break;
								}
							}
						}
					}
				}
			}
			return flag;
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return false;
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return false;
		}
	}

	public String getScenarioContent(String scenarioName) {
	    try(BufferedReader br = new BufferedReader(new FileReader(new File(scenarioDir, scenarioName + ".txt")))){
	        StringBuilder sb = new StringBuilder();
	        String line = br.readLine();
	        while (line != null) {
	            sb.append(line);
	            line = br.readLine();
	        }
	        return sb.toString().trim();
	    } catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return "";
		}
	}
	
	public String getEditableScenarioContent(String scenarioName) {
	    try(BufferedReader br = new BufferedReader(new FileReader(new File(scenarioDir, scenarioName + ".txt")))){
	        StringBuilder sb = new StringBuilder();
	        String line = br.readLine();
	        while (line != null) {
	        	if(!line.contains("@")){
	        		sb.append(line);
	        		sb.append("\n");
	        	}
	        	line = br.readLine();
	        }
	        return sb.toString().trim();
	    } catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return "";
		}
	}


	public void updateScenario(String scenarioName, String className, String editorContent, boolean status) {
		writePropertyHelper(className, updatePropertyHelper(scenarioName, className, status));
		saveScenario(scenarioName,className,editorContent);

	}

	public String updatePropertyHelper(String scenarioName, String className, boolean status) {
		String result = "";
		try (FileInputStream fis = new FileInputStream(this.initialize())) {
			prop.load(fis);
			if (!(prop.getProperty(className) == null)) {
				result = prop.getProperty(className);
				result = changeScenarioStatus(result, scenarioName, status);
			} else {
				result = scenarioName + ":" + status + ";";
			}
			return result;
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return "";
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return "";
		}
	}

	public String changeScenarioStatus(String oldProperty, String scenarioName, boolean status) {
		String result = "";
		String[] scenarioList = oldProperty.split(";");
		if (scenarioList.length > 0) {
			for (int i = 0; i < scenarioList.length; i++) {
				if (scenarioList[i].contains(scenarioName + ":")) {
					result = result + scenarioName + ":" + status + ";";
				} else {
					result = result + scenarioList[i] + ";";
				}
			}
		}
		return result;
	}

	public String deletePropertyHelper(String scenarioName, String className) {
		String result = "";
		try (FileInputStream fis = new FileInputStream(this.initialize())) {
			prop.load(fis);
			if (!(prop.getProperty(className) == null)) {
				result = prop.getProperty(className);
				result = deleteScenarioStatus(result, scenarioName);
			}
			return result;
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return "";
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return "";
		}
	}

	public String deleteScenarioStatus(String oldProperty, String scenarioName) {
		String result = "";
		String[] scenarioList = oldProperty.split(";");
		if (scenarioList.length > 0) {
			for (int i = 0; i < scenarioList.length; i++) {
				if (scenarioList[i].contains(scenarioName + ":")) {
					result = result;
				} else {
					result = result + scenarioList[i] + ";";
				}
			}
		}
		return result;
	}

	public void deleteScenario(String scenarioName, String className) {
		writePropertyHelper(className, deletePropertyHelper(scenarioName, className));
	}


}
