
package controller;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import Utils.ConstantsUtil;

public class ScenarioTranslationController {
	public static final ConfigController configController = new ConfigController();
	public static final ScenarioBindingController scenarioBindngController = new ScenarioBindingController();
	public static final JScenarioBindingController jscenarioBindingController = new JScenarioBindingController();
	public static final EventBindingController eventBindingController = new EventBindingController();
	public static final String JScenaioDir = configController.read(ConstantsUtil.JScenarioDirectory);
	public static final String ScenariosDir = "/Scenario";
	public static final String IUTContractDir = "/MonitoringFramework/IUTContract";
	public static final File Scenarios = new File(JScenaioDir, ScenariosDir);
	public static final File Contracts = new File(JScenaioDir, IUTContractDir);
	public static final File scenarioFile = scenarioBindngController.scenarioFile;
	public static final File ScenarioMetadataDir = scenarioBindngController.scenarioDir;
	public static final Properties prop = new Properties();
	public static final String IUTDir = "IUT";

	public void deleteContracts() {
		for (File f : Contracts.listFiles()) {
			f.delete();
		}
	}

	public void deleteScenarios() {
		for (File f : Scenarios.listFiles()) {
			f.delete();
		}
	}

	public void deleteAll() {
		deleteContracts();
		deleteScenarios();
	}

	public String headerBuilder() {
		return "package Scenario;\nimport java.util.ArrayList;\nimport java.util.Arrays;\nimport java.util.List;\nimport java.util.Map;\nimport org.apache.commons.collections4.list.SetUniqueList;\nimport org.apache.commons.lang3.tuple.Pair;\nimport MonitoringFramework.ACLClass.ACLContract;\nimport MonitoringFramework.ACLClass.ACLEventExpression;\nimport MonitoringFramework.ACLClass.ACLEventType;\nimport MonitoringFramework.ACLClass.ForTask;\nimport MonitoringFramework.ACLClass.ScenarioTask;\nimport MonitoringFramework.classtrn.EventToObserve;\nimport MonitoringFramework.classtrn.ObservabilityInformation;\nimport MonitoringFramework.classtrn.ScenarioParameter;\nimport MonitoringFramework.classtrn.StaticScenarioTaskType;\nimport Scenario.MyAnnotation.MyAnnotations.Scenario;\nimport MonitoringFramework.ACLClass.ChoiceAlternativeTask;\nimport MonitoringFramework.classtrn.ConditionToBeChecked;\nimport MonitoringFramework.classtrn.ObservabilityInformation;\nimport MonitoringFramework.classtrn.ConditionToBeChecked.ObservabilityType;\nimport MonitoringFramework.ACLClass.ParallelTask;\nimport MonitoringFramework.classtrn.ConditionToBeChecked.Operator;\n";
	}

	public String annotationBuilder(String scenarioName, String className, String trigger) {
		return "@Scenario(name=\"" + scenarioName + "\",owner=\"" + className + "\",trigger=\"" + trigger + "\")\n";
	}

	public String declarationBuilder(String scenarioName, String trigger) {
		return "public class " + scenarioName
				+ "Scenario extends MonitoringFramework.ACLClass.ACLScenario{\npublic static final String SELFNAME =\""
				+ scenarioName + "Scenario\";\npublic static final String TRIGGERNAME=\"" + trigger
				+ "\";\npublic static final Boolean HASPARAMETERS= true; \npublic static final List<String> EXPECTEDEVENTNAMES;\npublic "
				+ scenarioName
				+ "Scenario(ACLContract ownerContract,ACLEventType trigger) {\nsuper(ownerContract, trigger);\n}\n";
	}

	public String oneEventBuilderHelper(String event) {
		return "expectedEvents.add(\"" + event + "\");\n";
	}

	public String eventListBuilder(List<String> eventList) {
		String mainBody = "";
		for (String event : eventList) {
			mainBody = mainBody + oneEventBuilderHelper(event);
		}
		return "static{\nList<String> expectedEvents=new ArrayList<String>();\n" + mainBody
				+ "EXPECTEDEVENTNAMES = SetUniqueList.setUniqueList(expectedEvents);\n}\n";
	}

	public String singleTaskParamaterMapBuilder(Map<String, Integer> triggerParamaterMap, String task) {
		String taskParameter ="";
		if(task.indexOf("(")!=-1){
		taskParameter = task.substring(task.indexOf("(") + 1, task.indexOf(")"));
		}
		String[] taskParameters = taskParameter.split(",");
		String params = "";
		if (taskParameters.length >= 1) {
			for (String parameter : taskParameters) {
				Integer paramIndex = triggerParamaterMap.get(parameter.trim());
				if(paramIndex!=null){
				params = params + "parameterMap.get(TRIGGERNAME).get(" + paramIndex + "),";
				}
			}
		}
		return params;

	}

	public String singleLoopTaskParamaterMapBuilderHelper(Map<String, Integer> triggerParamaterMap, String task,
			String loopConditionMethod, Integer loopInstanceindex) {
		String params = singleTaskParamaterMapBuilder(triggerParamaterMap, task);
		params = params + "new ScenarioParameter(null, Pair.of(\"" + loopConditionMethod
				+ "\", parameterMap.get(TRIGGERNAME).get(" + loopInstanceindex + ")))";
		return params;

	}

	public String singleParameterMapBuilderHelper(Map<String, Integer> triggerParamaterMap, String taskContent) {
		String result = "";
		if (isForLoop(taskContent)) {
			String loopContent = taskContent.substring(taskContent.indexOf("{") + 1, taskContent.indexOf("}"));
			String[] loopEvents = loopContent.split("\\),");
			String loopCondition = taskContent.substring(0, taskContent.indexOf("{"))
					.substring(taskContent.indexOf("(") + 1, taskContent.indexOf(")"));
			String loopInstance = loopCondition.substring(0, loopCondition.lastIndexOf(".")).trim();
			Integer loopInstanceindex = triggerParamaterMap.get(loopInstance);
			String loopConditionMethod = loopCondition
					.substring(loopCondition.lastIndexOf(".") + 1, loopCondition.length()).trim();
			// check each paramater with the triggerMap and determine the index
			for (String event : loopEvents) {
				String params = singleLoopTaskParamaterMapBuilderHelper(triggerParamaterMap, event, loopConditionMethod,
						loopInstanceindex);
				result = result + "parameterMap.put(\""
						+ event.substring(event.indexOf("'") + 1, event.lastIndexOf("'"))
						+ "\", new ArrayList<>(Arrays.asList(" + params + ")));\n";
			}

			return result;

		} else if (isChoiceALternative(taskContent)) {
			String[] choiceContents = taskContent.split("}");
			for (String choice : choiceContents) {
				choice = choice.substring(choice.indexOf("{"));
				String[] events = choice.split("\\),");
				for (String event : events) {
					String params = singleTaskParamaterMapBuilder(triggerParamaterMap, event + ")");
					if (params.length() > 2) {
						result = result + "parameterMap.put(\""
								+ event.substring(event.indexOf("'") + 1, event.lastIndexOf("'"))
								+ "\", new ArrayList<>(Arrays.asList(" + params.substring(0, params.length() - 2)
								+ "))));\n";
					}
				}
			}
			return result;

		} else if (isParallel(taskContent)) {
			String[] parallelContents = taskContent.split("}");
			for (String choice : parallelContents) {
				choice = choice.substring(choice.indexOf("{"));
				String[] events = choice.split("\\),");
				for (String event : events) {
					String params = singleTaskParamaterMapBuilder(triggerParamaterMap, event + ")");
					result = result + "parameterMap.put(\""
							+ event.substring(event.indexOf("'") + 1, event.lastIndexOf("'"))
							+ "\", new ArrayList<>(Arrays.asList(" + params.substring(0, params.length() - 2)
							+ "))));\n";
				}
			}
			return result;

		} else {
			String params = singleTaskParamaterMapBuilder(triggerParamaterMap, taskContent);
			if (params.length() > 2) {
				return "parameterMap.put(\""
						+ taskContent.substring(taskContent.indexOf("'") + 1, taskContent.lastIndexOf("'"))
						+ "\", new ArrayList<>(Arrays.asList(" + params.substring(0, params.length() - 2) + "))));\n";
			}
		}
		return result;
	}

	public boolean isForLoop(String taskContent) {
		return taskContent.contains(ConstantsUtil.LOOP);
	}

	public boolean isChoiceALternative(String taskContent) {
		return taskContent.contains(ConstantsUtil.CHOICE);
	}

	public boolean isParallel(String taskContent) {
		return taskContent.contains(ConstantsUtil.PARALLEL);
	}

	public String triggerParameterMapBuilderHelper(Integer triggerParamaterMapLength) {
		String params = "";
		for (int i = 0; i < triggerParamaterMapLength; i++) {
			params = params + "new ScenarioParameter(null),";
		}
		return "parameterMap.put(TRIGGERNAME, new ArrayList<>(Arrays.asList(" + params.substring(0, params.length() - 2)
				+ "))));\n";
	}

	public int getParameterNumber(String className, String trigger) {
		// look up with event name instead of className+event name
		String realClassName = jscenarioBindingController.classLookUp(trigger);
		String triggerStr = eventBindingController.getFireEventImplementation(realClassName, trigger);
		return triggerStr.split(",").length - 1;
	}

	// public String parameterMapBuilder(String className, String trigger,
	// String[] taskContents) {
	// String parameterMap = parameterMapBuilder(getClassName(scenarioName),
	// getTriggerEvent(scenarioName),
	// getTaskContents(scenarioName));
	public Map<String, Integer> triggerParameterMapBuilder(String trigger) {
		Map<String, Integer> triggerParameterMap = new HashMap<String, Integer>();
		String parameters = trigger.substring(trigger.indexOf("(") + 1, trigger.indexOf(")"));
		String[] paramaterNames = parameters.split(",");
		for (int i = 0; i < paramaterNames.length; i++) {
			triggerParameterMap.put(paramaterNames[i].trim(), i);
		}
		return triggerParameterMap;
	}

	public Integer triggerParameterMapLength(String trigger) {
		String parameters = trigger.substring(trigger.indexOf("(") + 1, trigger.indexOf(")"));
		String[] paramaterNames = parameters.split(",");
		return paramaterNames.length;
	}

	public static void main(String[] args) {
		String loopCondition = "d.get";
		System.out.println(loopCondition.substring(0, loopCondition.lastIndexOf(".")).trim());
	}

	public String parameterMapBuilder(String scenarioName) {
		String[] taskContents = getTaskContents(scenarioName);
		String parameterMap = "@Override\nprotected void initializeParameterMap() {\n";
		// get trigger event paramater mapping,paramater name-index pair
		Map<String, Integer> triggerParameterMaps = triggerParameterMapBuilder(getWholeTriggerEvent(scenarioName));
		Integer triggerParamaterMapLength = triggerParameterMapLength(getWholeTriggerEvent(scenarioName));
		String triggerParameterMap = triggerParameterMapBuilderHelper(triggerParamaterMapLength);
		parameterMap = parameterMap + triggerParameterMap;
		for (String task : taskContents) {
			// int paramaterNumber = getParameterNumber(className, trigger);
			parameterMap = parameterMap + singleParameterMapBuilderHelper(triggerParameterMaps, task);
		}
		return parameterMap + "}\n";
	}
	
	public List<Integer> getObserverParameterIndexMap(String scenarioName, String conditionToBeCheck){
		List<Integer> observerParameterIndexMap=new ArrayList<Integer>();
		Map<String, Integer> triggerParameterMaps = triggerParameterMapBuilder(getWholeTriggerEvent(scenarioName));
		String conditionParameter=conditionToBeCheck.substring(conditionToBeCheck.indexOf("(") + 1, conditionToBeCheck.indexOf(")"))
				.trim();
		String[] paramaters=conditionParameter.split(",");
		if (paramaters.length >= 1) {
			for (String parameter : paramaters) {
				Integer paramIndex = triggerParameterMaps.get(parameter.trim());
				if(paramIndex!=null){
				observerParameterIndexMap.add(paramIndex);
				}
			}
		}
		return observerParameterIndexMap;
	}
	
	public String obseverParameterIndexMapBuilder(List<Integer> observerParameterIndexMap){
		String result="";
		for(Integer parameterIndex:observerParameterIndexMap){
			String temp="parameterMap.get(TRIGGERNAME).get("+parameterIndex+"),";
			result=result+temp;
		}
		result=result.substring(0, result.lastIndexOf(","));
		result="new ArrayList<Object>(Arrays.asList("+result+")),";
		return result;
	}

	public String singleLoopTaskBuilderHelper(int taskNumber, String taskContent, int observeInex,String scenarioName) {
		String taskName = "st" + taskNumber;
		String loopCondition = taskContent.substring(0, taskContent.indexOf("{"))
				.substring(taskContent.indexOf("(") + 1, taskContent.indexOf("){"));
		String loopContent = taskContent.substring(taskContent.indexOf("{") + 1, taskContent.indexOf("}"));
		String[] loopEvents = loopContent.split("\\),");
		//need to check if the loop condition has parameters, map the parameters
//		String loopConditionMethod = loopCondition.substring(loopCondition.lastIndexOf(".") + 1, loopCondition.length())
//				.trim();
		String loopConditionMethod = loopCondition.substring(loopCondition.lastIndexOf(".") + 1, loopCondition.lastIndexOf("("))
				.trim();
		List<Integer> observerParameterIndexMap=getObserverParameterIndexMap(scenarioName,loopCondition);
		String oberverPramater="";
		if(observerParameterIndexMap.size()>0){
			oberverPramater=obseverParameterIndexMapBuilder(observerParameterIndexMap);
		}else{
			oberverPramater="new ArrayList<Object>(),";
		}
		String taskDeclaration = "ForTask " + taskName + " = new ForTask(this, " + taskNumber
				+ ", new ObservabilityInformation(Pair.of(\"" + loopConditionMethod
				+ "\", parameterMap.get(TRIGGERNAME).get(" + observeInex + ")), "+oberverPramater+"ObservabilityType.VALUETYPE));\n";
		String forTasksParameterMapping = "";
		String forTasksMainBody = "";
		for (int i = 0; i < loopEvents.length; i++) {
			forTasksParameterMapping = forTasksParameterMapping + taskName + ".addEventParameterIndexInformation(\""
					+ loopEvents[i].substring(loopEvents[i].indexOf("'") + 1, loopEvents[i].lastIndexOf("'")) + "\", "
					+ observeInex + ");\n";
			if (i == 0) {
				forTasksMainBody = forTasksMainBody + "new EventToObserve(\""
						+ loopEvents[i].substring(loopEvents[i].indexOf("'") + 1, loopEvents[i].lastIndexOf("'"))
						+ "\", 1))";
			} else {
				forTasksMainBody = forTasksMainBody + ".and(new EventToObserve(\""
						+ loopEvents[i].substring(loopEvents[i].indexOf("'") + 1, loopEvents[i].lastIndexOf("'"))
						+ "\"))";
			}
		}
		String taskToRepeat = "ScenarioTask taskToBeRepeated;\nInteger numberOfTimesTaskisTobeRepeated = " + taskName
				+ ".getNumberOfForTaskConstituents();\n";
		String taskToRepeatLoop = "for (int i = 0; i < numberOfTimesTaskisTobeRepeated; i++) {\ntaskToBeRepeated = new StaticScenarioTaskType(this, "
				+ (taskNumber + 1) + ");\n";
		forTasksMainBody = "taskToBeRepeated.addACLEventProcessorToTask(new ACLEventExpression(taskToBeRepeated, EventProcessorState.CREATED).observeEvent("
				+ forTasksMainBody + ");\n" + taskName + ".setTaskToBeRepeated(taskToBeRepeated,i);\n}\n";
		String footer = "getConstituentsEventProcessors().add(" + taskName + ");\n";
		return taskDeclaration + forTasksParameterMapping + taskToRepeat + taskToRepeatLoop + forTasksMainBody + footer;
	}

	public String singleChoiceAlternativeTaskBuilderHelper(int taskNumber, String taskContent, int observeInex,String scenarioName) {
		String taskName = "st" + taskNumber;
		//need to check if the loop condition has parameters, map the parameters
		String choiceCondition = taskContent.substring(0, taskContent.indexOf("{"))
				.substring(taskContent.indexOf("(") + 1, taskContent.indexOf("){"));
		String choiceBlockContent = taskContent.substring(taskContent.indexOf("{"), taskContent.indexOf("}"));
		String[] choiceBlockEvents = choiceBlockContent.split("\\),");
		String alternativeBlockContent = taskContent.substring(taskContent.lastIndexOf("{"),
				taskContent.lastIndexOf("}"));
		String[] alternativeBlockEvents = alternativeBlockContent.split("\\),");
		String[] conidtionExtracted = choiceCondition.split(" ");
//		String conditionName = conidtionExtracted[0]
//				.substring(conidtionExtracted[0].lastIndexOf(".") + 1, conidtionExtracted[0].length()).trim();
//		;
		String conditionName = conidtionExtracted[0]
				.substring(conidtionExtracted[0].lastIndexOf(".") + 1, conidtionExtracted[0].lastIndexOf("(")).trim();
		;
		String operator = conidtionExtracted[1];
		String expectedCondition = conidtionExtracted[2];
		String operationType = "";
		if (expectedCondition.equals("true") || expectedCondition.equals("false")) {
			operationType = "ObservabilityType.BOOLEANTYPE";
		} else {
			operationType = "ObservabilityType.VALUETYPE";
		}
		List<Integer> observerParameterIndexMap=getObserverParameterIndexMap(scenarioName,conidtionExtracted[0]);
		String oberverPramater="";
		if(observerParameterIndexMap.size()>0){
			oberverPramater=obseverParameterIndexMapBuilder(observerParameterIndexMap);
		}else{
			oberverPramater="new ArrayList<Object>(),";
		}
		String conditionToCheck = "ConditionToBeChecked " + conditionName
				+ "Condition =new ConditionToBeChecked(new ObservabilityInformation(Pair.of(\"" + conditionName
				+ "\", parameterMap.get(TRIGGERNAME).get(" + observeInex + ")), " + oberverPramater + operationType + "), "
				+ expectedCondition + ", Operator." + operator + ");\n";
		String defineTaskName = "ScenarioTask " + taskName + " = new ChoiceAlternativeTask(this, " + taskNumber + ", "
				+ conditionName + "Condition);\n";
		String choichBlock = "";
		String alternativeBlock = "";
		for (int i = 0; i < choiceBlockEvents.length; i++) {
			if (i == 0) {
				if (choiceBlockEvents[i].indexOf("'") != -1) {
				choichBlock = choichBlock
						+ "new EventToObserve(\"" + choiceBlockEvents[i]
								.substring(choiceBlockEvents[i].indexOf("'") + 1, choiceBlockEvents[i].lastIndexOf("'"))
						+ "\", 1))";
				}else{
					choichBlock=null+")";
				}
			} else {
				choichBlock = choichBlock + ".and(new EventToObserve(\"" + choiceBlockEvents[i].substring(
						choiceBlockEvents[i].indexOf("'") + 1, choiceBlockEvents[i].lastIndexOf("'")) + "\"))";
			}
		}
		for (int i = 0; i < alternativeBlockEvents.length; i++) {
			if (i == 0) {
				if (alternativeBlockEvents[i].indexOf("'") != -1) {
					alternativeBlock = alternativeBlock + "new EventToObserve(\""
							+ alternativeBlockEvents[i].substring(alternativeBlockEvents[i].indexOf("'") + 1,
									alternativeBlockEvents[i].lastIndexOf("'"))
							+ "\", 1))";
				}else{
					alternativeBlock=null+")";
				}
			} else {
				alternativeBlock = alternativeBlock + ".and(new EventToObserve(\""
						+ alternativeBlockEvents[i].substring(alternativeBlockEvents[i].indexOf("'") + 1,
								alternativeBlockEvents[i].lastIndexOf("'"))
						+ "\"))";
			}
		}

		choichBlock = "((ChoiceAlternativeTask)" + taskName + ").setChoiceTask(new ACLEventExpression(" + taskName
				+ ", EventProcessorState.CREATED).observeEvent(" + choichBlock + ");\n";
		alternativeBlock = "((ChoiceAlternativeTask)" + taskName + ").setAlternativeTask(new ACLEventExpression("
				+ taskName + ", EventProcessorState.CREATED).observeEvent(" + alternativeBlock + ");\n";
		String footer = "getConstituentsEventProcessors().add(" + taskName + ");\n";
		return conditionToCheck + defineTaskName + choichBlock + alternativeBlock + footer;
	}

	public String singleParallelTaskBuilderHelper(int taskNumber, String taskContent) {
		String taskName = "st" + taskNumber;
		String parallelTask = "ParallelTask " + taskName + " = new ParallelTask(this, " + taskNumber + ");\n";
		String[] parallelBlocks = taskContent.split("}");
		String parallelBlock = "";
		String addParallelTask = "";
		for (int i = 0; i < parallelBlocks.length; i++) {
			String[] blockEvents = parallelBlocks[i].split("\\),");
			String mainBody = "";
			for (int j = 0; j < blockEvents.length; j++) {
				if (j == 0) {
					mainBody = mainBody + "new EventToObserve(\""
							+ blockEvents[j].substring(blockEvents[j].indexOf("'") + 1, blockEvents[j].lastIndexOf("'"))
							+ "\", 1))";
				} else {
					mainBody = mainBody + ".and(new EventToObserve(\""
							+ blockEvents[j].substring(blockEvents[j].indexOf("'") + 1, blockEvents[j].lastIndexOf("'"))
							+ "\"))";
				}
			}
			parallelBlock = parallelBlock + "ScenarioTask parallelTask" + i + "  = new StaticScenarioTaskType(this, "
					+ (taskNumber + i + 1) + ");\nparallelTask" + i
					+ ".addACLEventProcessorToTask(new ACLEventExpression(parallelTask" + i
					+ ", EventProcessorState.CREATED).observeEvent(" + mainBody + ");\n";
			addParallelTask = addParallelTask + ".addAParallelTask(parallelTask" + i + ")";
		}
		addParallelTask = "((ParallelTask)" + taskName + ")" + addParallelTask + ";\n";
		String footer = "getConstituentsEventProcessors().add(" + taskName + ");\n";
		return parallelTask + parallelBlock + addParallelTask + footer;
	}

	public String singleTasksBuilderHelper(int taskNumber, String taskContent, Integer observeIndex,String scenarioName) {
		if (isForLoop(taskContent)) {
			return singleLoopTaskBuilderHelper(taskNumber, taskContent, observeIndex,scenarioName);

		} else if (isChoiceALternative(taskContent)) {
			return singleChoiceAlternativeTaskBuilderHelper(taskNumber, taskContent, observeIndex,scenarioName );

		} else if (isParallel(taskContent)) {
			return singleParallelTaskBuilderHelper(taskNumber, taskContent);

		} else {
			String taskName = "st" + taskNumber;
			return "ScenarioTask " + taskName + " = new StaticScenarioTaskType(this," + taskNumber + ");\n" + taskName
					+ ".addACLEventProcessorToTask(new ACLEventExpression(" + taskName
					+ ", EventProcessorState.CREATED).observeEvent(new EventToObserve(\""
					+ taskContent.substring(taskContent.indexOf("'") + 1, taskContent.lastIndexOf("'"))
					+ "\",1)));\ngetConstituentsEventProcessors().add(" + taskName + ");\n";
		}
	}

	public Integer getObserveIndex(String scenarioName) {
		Map<String, Integer> triggerParameterMaps = triggerParameterMapBuilder(getWholeTriggerEvent(scenarioName));
		String[] taskContents = getTaskContents(scenarioName);
		Integer observeIndex = 0;
		for (String taskContent : taskContents) {
			if (isForLoop(taskContent)) {
				//need to check if the loop condition has parameters, map the parameters
				String loopCondition = taskContent.substring(0, taskContent.indexOf("{"))
						.substring(taskContent.indexOf("(") + 1, taskContent.indexOf(")"));
				String loopInstance = loopCondition.substring(0, loopCondition.lastIndexOf(".")).trim();
				observeIndex = triggerParameterMaps.get(loopInstance);
				break;
			} else if (isChoiceALternative(taskContent)) {
				//need to check if the loop condition has parameters, map the parameters
				String choiceCondition = taskContent.substring(0, taskContent.indexOf("{"))
						.substring(taskContent.indexOf("(") + 1, taskContent.indexOf(")"));
				String observeInstance = choiceCondition.substring(0, choiceCondition.lastIndexOf(".")).trim();
				observeIndex = triggerParameterMaps.get(observeInstance);
				break;
			}
		}
		return observeIndex;
	}

	public String taskBuilder(String scenarioName) {
		String[] taskContents = getTaskContents(scenarioName);
		Integer observeIndex = getObserveIndex(scenarioName);
		// block number not euqal to task number
		String taskInitialization = "@Override\nprotected void initializeTasksForScenario(){\n";
		int j = 0;
		for (int i = 0; i < taskContents.length; i++) {
			taskInitialization = taskInitialization + singleTasksBuilderHelper(j + 1, taskContents[i], observeIndex,scenarioName);
			if (isForLoop(taskContents[i])) {
				j = j + 2;
			} else if (isParallel(taskContents[i])) {
				j = j + taskContents[i].split(("}")).length + 1;
			} else {
				j = j + 1;
			}
		}
		return taskInitialization + "}\n";
	}

	public String footerBuilder() {
		return "@Override\npublic String getListenerName() {\nreturn SELFNAME;\n}\n@Override\nprotected Map<String, List<ScenarioParameter>> getParameterMap() {\nreturn this.parameterMap;\n}\n@Override\nprotected Boolean scenarioHasParameterizedEvents() {\nreturn HASPARAMETERS;\n}\n@Override\npublic List<String> getExpectedEvents() {\nreturn EXPECTEDEVENTNAMES;\n}\n@Override\nprotected String getExpectedTriggeringEventName() {\nreturn TRIGGERNAME;\n}\n}\n";
	}

	public String[] getTaskContents(String scenarioName) {
		String scenarioContent = scenarioBindngController.getScenarioContent(scenarioName).trim();
		String[] temp = scenarioContent.split(";");
		List<String> tempList = new ArrayList<String>();
		for (int i = 2; i < temp.length; i++) {
			tempList.add(temp[i]);
		}
		return (String[]) tempList.toArray(new String[tempList.size()]);
	}

	public String getTriggerEvent(String scenarioName) {
		String scenarioContent = scenarioBindngController.getScenarioContent(scenarioName);
		String tempTrigger = scenarioContent.split(";")[1];
		return tempTrigger.substring(tempTrigger.indexOf("(") + 1, tempTrigger.lastIndexOf("(")).trim();
	}

	public String getWholeTriggerEvent(String scenarioName) {
		String scenarioContent = scenarioBindngController.getScenarioContent(scenarioName);
		String tempTrigger = scenarioContent.split(";")[1];
		return tempTrigger.substring(tempTrigger.indexOf("(") + 1, tempTrigger.lastIndexOf(")"));
	}

	public List<String> getEventList(String scenaroName) {
		List<String> eventList = new ArrayList<String>();
		String[] eventContents = getTaskContents(scenaroName);
		for (String event : eventContents) {
			if (event.split("\\),").length > 1) {
				String[] subEventContents = event.split("\\),");
				for (String subEvent : subEventContents) {
					if (subEvent.contains("{") && subEvent.contains("}")) {
						// check for choice/alternative tasks
						String[] subSubEvents = subEvent.split("}");
						if (subSubEvents.length > 1) {
							if(subSubEvents[0].indexOf("'")!=-1){
							eventList.add(subSubEvents[0].substring(subSubEvents[0].indexOf("'") + 1,
									subSubEvents[0].lastIndexOf("'")));
							}
							if(subSubEvents[1].indexOf("'")!=-1){
							eventList.add(subSubEvents[1].substring(subSubEvents[1].indexOf("'") + 1,
									subSubEvents[1].lastIndexOf("'")));
							}
						} else {
							if(subSubEvents[0].indexOf("'")!=-1){
							eventList.add(subSubEvents[0].substring(subSubEvents[0].indexOf("'") + 1,
									subSubEvents[0].lastIndexOf("'")));
							}
						}
					} else {
						eventList.add(subEvent.substring(subEvent.indexOf("'") + 1, subEvent.lastIndexOf("'")));
					}
				}
			} else if (event.contains("{") && event.contains("}")) {
				// check for parallel tasks
				String[] subSubEvents = event.split("}");
				if (subSubEvents.length > 1) {
					if (subSubEvents[0].indexOf("'") != -1 && subSubEvents[0].lastIndexOf("'") != -1) {
						eventList.add(subSubEvents[0].substring(subSubEvents[0].indexOf("'") + 1,
								subSubEvents[0].lastIndexOf("'")));
					}
					if (subSubEvents[1].indexOf("'") != -1 && subSubEvents[1].lastIndexOf("'") != -1) {
						eventList.add(subSubEvents[1].substring(subSubEvents[1].indexOf("'") + 1,
								subSubEvents[1].lastIndexOf("'")));
					}
				} else {
					if (subSubEvents[0].indexOf("'") != -1 && subSubEvents[0].lastIndexOf("'") != -1) {
						eventList.add(subSubEvents[0].substring(subSubEvents[0].indexOf("'") + 1,
								subSubEvents[0].lastIndexOf("'")));
					}
				}
			} else if (event.contains("'")) {
				eventList.add(event.substring(event.indexOf("'") + 1, event.lastIndexOf("'")));
			}
		}
		return eventList;
	}

	public String getContractName(String scenarioName) {
		String scenarioContent = scenarioBindngController.getScenarioContent(scenarioName);
		String temp = scenarioContent.split(";")[0];
		String className = temp.split(",")[1].substring(temp.split(",")[1].lastIndexOf("=") + 1,
				temp.split(",")[1].length());
		String[] classNametemp = className.split("\\.");
		if (classNametemp.length > 1) {
			String realClassName = classNametemp[classNametemp.length - 1];
			return realClassName + "Contract";
		} else {
			return className + "Contract";
		}

	}

	public String getClassName(String scenarioName) {
		String scenarioContent = scenarioBindngController.getScenarioContent(scenarioName);
		String temp = scenarioContent.split(";")[0];
		String className = temp.split(",")[1].substring(temp.split(",")[1].lastIndexOf("=") + 1,
				temp.split(",")[1].length());
		return className;
	}

	public String translateScenario(String scenarioName) {
		String header = headerBuilder();
		String annotation = annotationBuilder(scenarioName, getContractName(scenarioName),
				getTriggerEvent(scenarioName));
		String declaration = declarationBuilder(scenarioName, getTriggerEvent(scenarioName));
		String eventList = eventListBuilder(getEventList(scenarioName));
		String parameterMap = parameterMapBuilder(scenarioName);
		String tasks = taskBuilder(scenarioName);
		String footer = footerBuilder();
		return header + annotation + declaration + eventList + parameterMap + tasks + footer;
	}

	public void publishTranslation(String scenarioName) {
		String file = translateScenario(scenarioName);
		File scenarioFile = new File(Scenarios, File.separator + scenarioName + "Scenario.java");
		if (!scenarioFile.exists()) {
			try {
				scenarioFile.createNewFile();
				try (FileWriter writer = new FileWriter(scenarioFile, false);) {
					writer.write(file);
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		} else {
			try (FileWriter writer = new FileWriter(scenarioFile, false);) {
				writer.write(file);
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}

	public void translateAll() {
		// register all active scenarios
		jscenarioBindingController.scenarioUtilsBuilder();
		// transalate each scenarios
		List<String> activeScenarioList = jscenarioBindingController.getActiveScenarioList();
		for (String scenario : activeScenarioList) {
			publishTranslation(scenario);
		}
	}

	public String contractHeaderBuilder(String contract) {
		return "package MonitoringFramework.IUTContract;\nimport java.util.ArrayList;\nimport java.util.HashMap;\nimport java.util.List;\nimport java.util.Map;\nimport org.apache.commons.collections4.list.SetUniqueList;\nimport MonitoringFramework.ACLClass.ACLContract;\nimport Scenario.Utility.StringUtils;\nimport "
				+ IUTDir + "." + contract + ";\n";
	}

	public String contractDeclarationBuilder(String className) {// after
																// processed
																// className,the
																// last part
																// after.
		String contractName = className + "Contract";
		return "public class " + contractName
				+ " extends ACLContract {\nprivate Object iutType;\npublic static final List<String> EXPECTEDEVENTNAMES;\npublic static final Map<String, List<String>> SCENARIOTRIGGERMAP;\npublic static final String SELFNAME =\""
				+ contractName + "\";\npublic " + contractName + "(" + className
				+ " iutType) {\nsuper();\nthis.iutType = iutType;\n}\npublic " + contractName + "() {}\n";
	}

	public String contractFooterBuilder(String className) {
		String contractName = className + "Contract";
		return "static{\nList<Class<?>> scenarioList=StringUtils.getScenarioListByContractName(SELFNAME);\nList<String> triggerEvents=StringUtils.getAllTriggerEvents(scenarioList);\nEXPECTEDEVENTNAMES = SetUniqueList.setUniqueList(triggerEvents);\nSCENARIOTRIGGERMAP = new HashMap<String, List<String>>();\nEXPECTEDEVENTNAMES.forEach((eventName) -> SCENARIOTRIGGERMAP.put(eventName, getScenarioNamesbyTriggeringEvent(eventName,SELFNAME)));\n}\npublic static List<String> getScenarioNamesbyTriggeringEvent(String triggeringEvent,String contract){\nList<String> scenarioNameList = new ArrayList<>();\nscenarioNameList=StringUtils.getAllScenarioNames(triggeringEvent,contract);\nreturn scenarioNameList;\n}\n@Override\npublic void setBindpoint(Object instance) {\nthis.iutType = instance;\n	}\n@Override\npublic Object getBindpoint() {\nreturn this.iutType;\n}\n@Override\npublic Map<String, List<String>> getScenarioTriggerMap() {\nreturn SCENARIOTRIGGERMAP;\n}\n@Override\npublic List<String> getExpectedEventNames() {\nreturn EXPECTEDEVENTNAMES;\n}\n@Override\npublic String getListenerName() {\nreturn SELFNAME;\n}\n@Override\npublic int hashCode() {\nfinal int prime = 31;\nint result = super.hashCode();\nresult = prime * result + ((iutType == null) ? 0 : iutType.hashCode());\nreturn result;\n}\n@Override\npublic boolean equals(Object obj) {\nif (this == obj) {\nreturn true;\n}\nif (!super.equals(obj)) {\nreturn false;\n}\nif (!(obj instanceof "
				+ contractName + ")) {\nreturn false;\n}\nreturn true;\n}\n}\n";

	}

	public String translateContract(String contract) {
		return contractHeaderBuilder(contract) + contractDeclarationBuilder(processClassName(contract))
				+ contractFooterBuilder(processClassName(contract));
	}

	public String processClassName(String contract) {
		String[] classTemps = contract.split("\\.");
		if (classTemps.length > 1) {
			return classTemps[classTemps.length - 1];
		} else {
			return contract;
		}
	}

	public void publishContract(String contract) {
		String file = translateContract(contract);
		File contractFile = new File(Contracts, File.separator + processClassName(contract) + "Contract.java");
		if (!contractFile.exists()) {
			try {
				contractFile.createNewFile();
				try (FileWriter writer = new FileWriter(contractFile, false)) {
					writer.write(file);
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		} else {
			try (FileWriter writer = new FileWriter(contractFile, false)) {
				writer.write(file);
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}

	}

	public void buildAllContract() {
		List<String> activeContractList = jscenarioBindingController.getActiveContractList();
		for (String contract : activeContractList) {
			publishContract(contract);
		}
	}

}
