package controller;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.List;

import TestSuites.ScenarioTester;
import Utils.ConstantsUtil;

public class TestSuitesController {
	public static final ConfigController configController = new ConfigController();
	public static final ScenarioTranslationController scenarioTranslationController = new ScenarioTranslationController();
	public static final String JScenaioDir = configController.read(ConstantsUtil.JScenarioDirectory);
	public static final String TestSuitesDir = "/TestSuites";
	public static final String TestReportDir = "/TestReport";
	public static final File contractsDir = new File(JScenaioDir, "/MonitoringFramework/IUTContract");
	public static final File Tester = new File(JScenaioDir, TestSuitesDir);
	public static final File Reporter = new File(JScenaioDir, TestReportDir);
	public static final File scenarioTester = new File(Tester, "ScenarioTester.java");
	public static final File systemSimulation = new File(Tester, "SystemSimulation.java");
	public static final File fullReport = new File(Reporter, "FullReport.txt");
	public static final String IUTDir="IUT";
	
	public void deleteScenarioTester(){
		String file = testerTemplate();
		if (!scenarioTester.exists()) {
			try {
				scenarioTester.createNewFile();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		try (FileWriter writer = new FileWriter(scenarioTester, false);) {
			writer.write(file);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public void deleteSystemSimulation(){
		String file = simulatorTemplate();
		if (!systemSimulation.exists()) {
			try {
				systemSimulation.createNewFile();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		try (FileWriter writer = new FileWriter(systemSimulation, false);) {
			writer.write(file);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public void deleteFullReport(){
		fullReport.delete();
	}

	public void deleteAll(){
		deleteScenarioTester();
		deleteSystemSimulation();
		deleteFullReport();
	}

	public List<String> getAllContract() {
		List<String> contractList = new ArrayList<String>();
		File[] listOfFiles = contractsDir.listFiles();
		for (int i = 0; i < listOfFiles.length; i++) {
			if (listOfFiles[i].isFile()) {
				contractList.add(listOfFiles[i].getName().substring(0, listOfFiles[i].getName().lastIndexOf(".")));
			}
		}
		return contractList;
	}

	public String importerBuilder(List<String> contractList) {
		String result = "";
		for (String contract : contractList) {
			String importer = "import MonitoringFramework.IUTContract." + contract + ";\n";
			result = result + importer;
		}
		result = result
				+ "import MonitoringFramework.classtrn.BlackBoard;\nimport MonitoringFramework.classtrn.Constants;\n";
		return result;
	}

	public String testerHeaderBuilder() {
		String result = "";
		result = result + "package TestSuites;\n";
		result = result + importerBuilder(getAllContract());
		return result + "public class ScenarioTester {\npublic static void main(String[] args) {\n";
	}

	public String contractInitializationBuilder(List<String> contractList) {
		String result = "";
		for (String contract : contractList) {
			String contractName = scenarioTranslationController.processClassName(contract);
			String initializer = contractName + " " + contractName.toLowerCase() + "=new " + contractName + "();\n"
					+ contractName.toLowerCase() + ".registerForACLEvents();\n";
			result = result + initializer;
		}
		return result;
	}

	public String methodBodyBuilder() {
		return "SystemSimulation.MyIUTPoolSimulation();\nMonitoringFramework.classtrn.BlackBoard.getInstance().notifyIUTExit();\nwhile(!BlackBoard.getInstance().isTimeToshutDownEventNotification()){\ntry {\nThread.sleep(1000* Constants.WAITFOREVENTPROCESSING);\n} catch (InterruptedException e) {\ne.printStackTrace();\n}\n}\n";
	}

	public String exitMethodBodyBuilder(List<String> contractList) {
		String result = "";
		for (String contract : contractList) {
			String contractName = scenarioTranslationController.processClassName(contract);
			String exiter = contractName.toLowerCase() + ".getScenarioInstanceStatusAfterIUTExit();\n";
			result = result + exiter;
		}
		return result;
	}

	public String testerFooter() {
		return "}\n}\n";
	}

	public String testerBuiler() {
		List<String> contractList = getAllContract();
		return testerHeaderBuilder() + contractInitializationBuilder(contractList) + methodBodyBuilder()
				+ exitMethodBodyBuilder(contractList) + testerFooter();
	}
	
	public String testerTemplate(){
		return "package TestSuites;\npublic class ScenarioTester {\npublic static void main(String[] args) {\n}}\n";
	}

	public void publicTester() {
		String file = testerBuiler();
//		if (!scenarioTester.exists()) {
//			try {
//				scenarioTester.createNewFile();
//			} catch (IOException e) {
//				// TODO Auto-generated catch block
//				e.printStackTrace();
//			}
//		}
		try (FileWriter writer = new FileWriter(scenarioTester, false)) {
			writer.write(file);
			writer.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}

	public String simulatorTemplate(){
		return "package TestSuites;\n//please import your used class here\npublic class SystemSimulation {\npublic static void MyIUTPoolSimulation(){\n//please add your test plan here\n}\n}\n";
	}
	
	public void saveTestPlan(String editorContent) {
		try (FileWriter writer = new FileWriter(systemSimulation, false)) {
			writer.write(editorContent);
			writer.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public String getEditableTestPlanContent() {
		try (BufferedReader br = new BufferedReader(new FileReader(systemSimulation))) {
			StringBuilder sb = new StringBuilder();
			String line = br.readLine();
			while (line != null) {
				sb.append(line);
				sb.append("\n");
				line = br.readLine();
			}
			return sb.toString().trim();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return "";
		}
	}

	public static void testerRunner() throws Exception {
		// redirect console output to the file
		PrintStream out = new PrintStream(new FileOutputStream(fullReport));
		System.setOut(out);
		System.setErr(out);
		ScenarioTester.main(null);

	}

	public List<String> processReport() {
		List<String> reportLines = new ArrayList<String>();
		try (BufferedReader br = new BufferedReader(new FileReader(fullReport))) {
			String line = br.readLine();
			while (line != null) {
				if (line.contains("ScenarioName:") && line.contains("InstanceId:") && line.contains("final state:")) {
					reportLines.add(line);
				}
				line = br.readLine();
			}
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return reportLines;
	}

	public Object[][] getReportTable() {
		Object[][] result = null;
		List<String> reportLines = processReport();
		result = new Object[reportLines.size()][3];
		for (int i = 0; i < reportLines.size(); i++) {
			String[] strs = reportLines.get(i).split(":");
			String scenarioName = strs[1].substring(strs[1].indexOf(" "), strs[1].indexOf("InstanceId"));
			String instanceId = strs[2].substring(strs[2].indexOf(" "), strs[2].indexOf("Triggering"));
			String state = strs[strs.length - 1].trim();
			result[i] = new Object[] { scenarioName, instanceId, state };
		}
		return result;
	}

	public String getFullReportContent() {
		try (BufferedReader br = new BufferedReader(new FileReader(fullReport))) {
			 StringBuilder sb = new StringBuilder();
		        String line = br.readLine();
		        while (line != null) {
		        		sb.append(line);
		        		sb.append("\n");
		        	line = br.readLine();
		        }
		        return sb.toString().trim();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return "";
		}
	}
}
