package layout;

import java.awt.EventQueue;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.border.EmptyBorder;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.DefaultTableModel;
import controller.EventBindingController;
import controller.ScenarioBindingController;
import controller.ScenarioTranslationController;

public class ScenarioDefinition extends JFrame {

	private JPanel contentPane;
	private JTable table;
	private DefaultTableModel tableModel;
	private static final EventBindingController eventBindingController = new EventBindingController();
	private static final ScenarioBindingController scenarioBindingController=new ScenarioBindingController();
	private static final ScenarioTranslationController scenarioTranslationController=new ScenarioTranslationController();

	/**
	 * Launch the application.
	 */
	public static void main(String[] args) {
		EventQueue.invokeLater(new Runnable() {
			public void run() {
				try {
					ScenarioDefinition frame = new ScenarioDefinition();
					frame.setVisible(true);
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		});
	}

	/**
	 * Create the frame.
	 */
	public ScenarioDefinition() {
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		setBounds(200, 200, 850, 600);
		setTitle("JScenario");
		contentPane = new JPanel();
		contentPane.setBorder(new EmptyBorder(5, 5, 5, 5));
		setContentPane(contentPane);
		contentPane.setLayout(null);
		
		JLabel lblStep = new JLabel("Step 3 : Define Scenarios");
		lblStep.setFont(new Font("Times New Roman", Font.BOLD, 18));
		lblStep.setBounds(33, 38, 317, 32);
		contentPane.add(lblStep);

		JButton btnNextStep = new JButton("Next Step");
		btnNextStep.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				ExecutorService executorService = Executors.newSingleThreadExecutor();
				Runnable task = () -> {
					scenarioTranslationController.buildAllContract();
				};
				executorService.execute(task);
				Runnable task1 = () -> {
					scenarioTranslationController.translateAll();
				};
				executorService.execute(task1);
				Runnable task2 = () ->{
					setVisible(false);
					RunTestPlan sd = new RunTestPlan();
					sd.setVisible(true);
				};
				executorService.execute(task2);
				executorService.shutdown();

			}
		});
		btnNextStep.setFont(new Font("Times New Roman", Font.PLAIN, 18));
		btnNextStep.setBounds(671, 437, 123, 45);
		contentPane.add(btnNextStep);

		JButton btnPrevStep = new JButton("Prev Step");
		btnPrevStep.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				// go back to previous step
				setVisible(false);
				EventsDefinitionWithImport ed = new EventsDefinitionWithImport();
				ed.setVisible(true);
			}
		});
		btnPrevStep.setFont(new Font("Times New Roman", Font.PLAIN, 18));
		btnPrevStep.setBounds(33, 437, 123, 45);
		contentPane.add(btnPrevStep);
		
		JLabel lblBrowingByClass = new JLabel("Browsing existing scenarios by contractor :");
		lblBrowingByClass.setFont(new Font("Times New Roman", Font.PLAIN, 18));
		lblBrowingByClass.setBounds(33, 186, 330, 32);
		contentPane.add(lblBrowingByClass);
		
		
		
		//give value to the dropbox, need to be replaced by real data producer
		List<String> keys = eventBindingController.getAllProperty();
		String[] classStr = new String[keys.size()];
		classStr = keys.toArray(classStr);
		JComboBox comboBox = new JComboBox(classStr);
		comboBox.setFont(new Font("Times New Roman", Font.PLAIN, 18));
		comboBox.setBounds(342, 191, 452, 22);
		contentPane.add(comboBox);
		//add listen to handle the table change to refresh the table
		comboBox.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				JComboBox cb = (JComboBox) e.getSource();
				String className = (String) cb.getSelectedItem();
				String[] columnNames = {"Scenario","Status"};
				Object[][] data = scenarioBindingController.getTable(className);
				// refresh the table
				tableModel = new DefaultTableModel(data, columnNames){
					public boolean isCellEditable(int row, int column) {
						return false;
					}
				};
				table.setModel(tableModel);
		    }
		});
		
		
		//get table date,need to be replaced by real date producer
		String className = (String) comboBox.getSelectedItem();
		String[] columnNames = {"Scenario","Status"};
		Object[][] data = scenarioBindingController.getTable(className);
		tableModel = new DefaultTableModel(data, columnNames) {
			public boolean isCellEditable(int row, int column) {
				return false;
			}
		};
		table = new JTable(tableModel);
		table.setFont(new Font("Times New Roman", Font.PLAIN, 18));
		table.getTableHeader().setFont(new Font("Times New Roman", Font.PLAIN, 18));
		table.setBounds(264, 307, 1, 1);
		table.setRowHeight(25);
		
		table.addMouseListener(new MouseAdapter() {
		    @Override
		    public void mouseClicked(MouseEvent evt) {
		        int row = table.rowAtPoint(evt.getPoint());
		        int col = table.columnAtPoint(evt.getPoint());
		        if (row >= 0 && col >= 0) {
		        	//on click row open the dialog to modify the data, get both contractor name and scenario name for modifying
		        	String className=comboBox.getSelectedItem().toString();
		        	String scenarioName=table.getValueAt(table.getSelectedRow(), 0).toString();
		        	ScenarioEditor editor=new ScenarioEditor(className,scenarioName);
		        	editor.setVisible(true);
		        	setVisible(false);
		        }
		    }
		});
		contentPane.add(table);
		
		JScrollPane scrollPane = new JScrollPane(table);
		scrollPane.setBounds(33, 231, 761, 180);
		contentPane.add(scrollPane);
		
		JButton btnNewButton = new JButton("Create");
		btnNewButton.setFont(new Font("Times New Roman", Font.PLAIN, 18));
		btnNewButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				ScenarioEditor editor=new ScenarioEditor();
	        	editor.setVisible(true);
	        	setVisible(false);
			}
		});
		btnNewButton.setBounds(33, 121, 123, 32);
		contentPane.add(btnNewButton);

	}

}
